@@ -9,16 +9,13 @@ AUTHOR:        kingson fiasco
  This file may be used, distributed and modified without limitation.
  *******************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#else 
-#define VERSION	"0.2.0"
-#define QTERM_DATADIR "/usr/share/qterm"
-#define QTERM_BINDIR  "/usr/bin"
-#endif
 
 // remove this when use configure
 
+#include "qterm.h"
+#ifdef HAVE_PYTHON
+#include <Python.h>
+#endif
 #include <qpixmap.h>
 #include <qstringlist.h>
 #include <qapplication.h>
@@ -26,125 +23,271 @@ AUTHOR:        kingson fiasco
 #include "qtermconfig.h"
 #include "qtermparam.h"
 
-#ifndef _OS_WIN32_
-#include <sys/stat.h>
 #include <stdlib.h>
+#if !defined(_OS_WIN32_) && !defined(Q_OS_WIN32)
+#include <sys/stat.h>
 #include <errno.h>
 #include <locale.h>
 #endif
 #include <qdir.h>
 #include <qfile.h>
+#include <qfileinfo.h>
 
-char fileCfg[128]="./qterm.cfg";
-char addrCfg[128]="./address.cfg";
+#include <qmessagebox.h>
+#include <qfiledialog.h>
+#include <qobject.h>
+#include <qwidget.h>
+
+QString fileCfg="./qterm.cfg";
+QString addrCfg="./address.cfg";
 
 QString pathLib="./";
 QString pathPic="./";
+QString pathCfg="./";
 
-#ifndef _OS_WIN32_
+void runProgram(const QCString& cmd)
+{
+	QCString cstrCmd=cmd;
+	#if !defined(_OS_WIN32_) && !defined(Q_OS_WIN32)
+	cstrCmd += " &";
+	#endif
+	system(cstrCmd);
+}
+
+QString getOpenFileName(const QString& filter, QWidget * widget)
+{
+	QTermConfig conf(fileCfg);
+	QString path = QString::fromLocal8Bit(conf.getItemValue("global","openfiledialog"));
+
+	QString strOpen = QFileDialog::getOpenFileName(	path, filter, widget);
+
+	if(!strOpen.isEmpty())
+	{
+		// save the path
+		QFileInfo fi(strOpen);
+		conf.setItemValue("global","openfiledialog",fi.dirPath(true));
+		conf.save(fileCfg);
+	}
+	
+	return strOpen;
+}
+
+// warning for overwrite getSaveFileName
+QString getSaveFileName(const QString& filename, QWidget* widget)
+{
+	// get the previous dir
+	QTermConfig conf(fileCfg);
+	QString path = QString::fromLocal8Bit(conf.getItemValue("global","savefiledialog"));
+
+	QString strSave = QFileDialog::getSaveFileName(	path+"/"+G2U(filename), "*", widget);
+
+	QFileInfo fi(strSave);
+	
+	while(fi.exists())
+	{
+		int yn = QMessageBox::warning(widget, "QTerm", 
+					"File exists. Overwrite?", "Yes", "No");
+		if(yn==0)
+			break;
+		strSave = QFileDialog::getSaveFileName(path+"/"+G2U(filename), "*", widget);
+		if(strSave.isEmpty())
+			break;
+	}
+
+	if(!strSave.isEmpty())
+	{
+		// save the path
+		conf.setItemValue("global","savefiledialog",fi.dirPath(true));
+		conf.save(fileCfg);
+	}
+
+	return strSave;
+}
+
+void clearDir( const QString& path )
+{
+	QDir dir(path);
+	if( dir.exists() )
+	{
+		const QFileInfoList *list = dir.entryInfoList();
+		QFileInfoListIterator it( *list );
+		QFileInfo *fi;
+		while ( (fi = it.current()) != 0 ) 
+		{
+			if(fi->isFile())
+				dir.remove(fi->fileName());
+			++it;
+		}
+	}
+}
+
+int checkPath( const QString& path )
+{
+	QDir dir(path);
+	if( ! dir.exists() )
+	{
+		if( !dir.mkdir(path) )
+		{
+			printf("Failed to create directory %s\n", (const char*)path.local8Bit());
+			return -1;
+		}
+	}
+	return 0;
+}
+
+int checkFile( const QString& dst, const QString& src )
+{
+	QDir dir;
+	if(dir.exists(dst))
+		return 0;
+	
+	if( !dir.exists(src) )
+	{
+		printf("QTerm failed to find %s.\n"
+			"Please copy the qterm.cfg from the source tarball to %s\n", 
+			(const char*)src.local8Bit(), (const char*) dst.local8Bit() );
+		return -1;
+	}
+		
+	QString cmd;
+	cmd="/bin/cp -f "+ src+" "+dst;
+
+	if ( system((const char *)cmd.local8Bit())==-1 )
+	{
+		printf("QTerm failed to %s. \n"
+				"Please copy the qterm.cfg from the source tarball to %s\n", 
+			(const char*)cmd.local8Bit(), (const char *)dst.local8Bit());
+		return -1;
+	}
+	return 0;
+}
+
+#if defined(_OS_WIN32_) || defined(Q_OS_WIN32)
+#ifndef MAX_PATH
+#define MAX_PATH 128
+#endif
+#include <windows.h>
+#include <shellapi.h>
 int iniWorkingDir( QString param )
 {
+    char ExeNamePath[MAX_PATH], _fileCfg[MAX_PATH], _addrCfg[MAX_PATH];
+    size_t LastSlash = 0;
+
+    if(0 == GetModuleFileNameA(NULL, ExeNamePath, MAX_PATH))
+    {
+        const char *DefaultModuleName = "./qterm.exe";
+        strcpy(ExeNamePath, DefaultModuleName);
+    }
+    for(size_t i = 0, ModuleNameLen = strlen(ExeNamePath) ;
+        i < ModuleNameLen ; ++i)
+    {
+        if(ExeNamePath[i] == '\\')
+        {
+            ExeNamePath[i] = '/';
+            LastSlash = i;
+        }
+    }
+    ExeNamePath[LastSlash+1] = '\0';
+    pathLib = QString::fromLocal8Bit(ExeNamePath);
+    pathPic = QString::fromLocal8Bit(ExeNamePath);
+    pathCfg = QString::fromLocal8Bit(ExeNamePath);
+    strcpy(_fileCfg, ExeNamePath);
+    strcat(_fileCfg, "qterm.cfg");
+    fileCfg = QString::fromLocal8Bit(_fileCfg);
+    strcpy(_addrCfg, ExeNamePath);
+    strcat(_addrCfg, "address.cfg");
+    addrCfg = QString::fromLocal8Bit(_addrCfg);
+
+	QString pathSchema=pathCfg+"schema";
+	if(checkPath(pathSchema)==-1)
+		return -1;
+
+}
+#else
+int iniWorkingDir( QString param )
+{
+	QDir dir;
+	QFileInfo fi;
+#ifdef Q_OS_MACX
+	// $HOME/Library/QTerm/
+	QString pathHome=QDir::homeDirPath();
+	pathCfg = pathHome+"/Library/QTerm/";
+	if(checkPath(pathCfg)==-1)
+		return -1;
+
+	// get executive file path 
+	fi.setFile(param);
+	pathLib=fi.dirPath()+'/';
+#else
+	pathCfg=QDir::homeDirPath()+"/.qterm/";
+	if(checkPath(pathCfg)==-1)
+		return -1;
+
 	// pathLib --- where datedir "pic", "cursor", "po"
 	if(param.find('/')==-1)
 		pathLib=QTERM_DATADIR"/";
 	else
 	{
-		param.truncate( param.findRev('/') );
-		QString oldPath=QDir::currentDirPath();
-		QDir::setCurrent( param );
-		if( QDir::currentDirPath()==QTERM_BINDIR )
-			pathLib=QTERM_DATADIR;
-		else
-			pathLib=QDir::currentDirPath();
-		QDir::setCurrent( oldPath );
-		pathLib+='/';
+        QFileInfo fi(param);
+        if( fi.isSymLink() )
+            param = fi.readLink();
+        // get the pathname
+        param.truncate( param.findRev('/') );
+        QString oldPath=QDir::currentDirPath();
+        QDir::setCurrent( param );
+        dir.setPath(QTERM_BINDIR);
+        if( dir == QDir::current() )
+            pathLib=QTERM_DATADIR;
+        else
+            pathLib=QDir::currentDirPath();
+        QDir::setCurrent( oldPath );
+        pathLib+='/';
 	}
+#endif
 
+	QString pathSchema=pathCfg+"schema";
+	if(checkPath(pathSchema)==-1)
+		return -1;
 
-	QDir dir;
-	
-	QString pathHome=QDir::homeDirPath()+"/.qterm";
-
-	dir.setPath( pathHome );
-	if( ! dir.exists() )
-	{
-		if( !dir.mkdir(pathHome) )
-		{
-			return -1;
-		}
-	}
 
 	// picPath --- $HOME/.qterm/pic prefered
-	pathPic = QDir::homeDirPath()+"/.qterm/pic";
+	pathPic = pathCfg+"pic";
 	dir.setPath( pathPic );
 	if( !dir.exists() )
 		pathPic = pathLib;
 	else
-		pathPic = pathHome+"/";
-
-	QString file=QDir::homeDirPath()+"/.qterm/qterm.cfg";
-	QString addr=QDir::homeDirPath()+"/.qterm/address.cfg";
-
-	sprintf( fileCfg,file.ascii(),file.length() );
-	sprintf( addrCfg,addr.ascii(),addr.length() );
-
+		pathPic = pathCfg;
 	
-	if( !dir.exists(file) || !dir.exists(addr) )
-	{
-		QString srcCfg=pathLib+"qterm.cfg";
-		if( !dir.exists( srcCfg ) )
-		{
-			printf(" Failed to find %s\n Make sure you have this file there\n"
-				"There may be two reasons for this:\n"
-				" 1. QTerm is not install properly\n"
-				" 2. You have a corrupted source package.\n"
-				"Solution:\n"
-				"Re-Get source tarball and copy the qterm.cfg manually\n", srcCfg.ascii() );
-			exit(-1);
-		}
-		
-		QString cmd;
-		cmd="/bin/cp -f "+ pathLib+"qterm.cfg ";
-		cmd+=pathHome;
-		
-
-		if ( system(cmd)==-1 )
-		{
-			printf(" Failed to %s \nMake sure you have qterm.cfg in %s\n"
-				"There may be two reasons for this:\n"
-				" 1. QTerm is not install properly\n"
-				" 2. You have a corrupted source package.\n"
-				"Solution:\n"
-				"Re-Get source tarball and copy the qterm.cfg manually\n",cmd.ascii(),pathLib.ascii());
-			exit(-1);
-		}
-
-		cmd="/bin/cp -f "+ pathLib+"address.cfg ";
-		cmd+=pathHome;
-		
-
-		if ( system(cmd)==-1 )
-		{
-			printf(" Failed to %s \nMake sure you have address.cfg in %s\n"
-				"There may be two reasons for this:\n"
-				" 1. QTerm is not install properly\n"
-				" 2. You have a corrupted source package.\n"
-				"Solution:\n Re-Get source tarball and copy the address.cfg manually\n",cmd.ascii(),pathLib.ascii());
-			exit(-1);
-		}
-
-
-	}
-
-	return 1;
+	// configuration files
+	fileCfg=pathCfg+"qterm.cfg";
+	if(checkFile(fileCfg,pathLib+"qterm.cfg")==-1)
+		return -1;
+	addrCfg=pathCfg+"address.cfg";
+	if(checkFile(addrCfg,pathLib+"address.cfg")==-1)
+		return -1;
+	
+	return 0;
 }
 #endif
 
-void iniSettings()
+int iniSettings()
 {
    //read settings from qterm.cfg
 	QTermConfig * conf= new QTermConfig(fileCfg);
 
+#if 0
+	// get the version info first
+	QString ver = conf->getItemValue("global","version");
+	if(ver.toInt()<0000300)
+	{
+		printf("It seems that you are still using the old formated configuration file. Please update first.\n"
+				"The simple way is to backup your directory ~/.qterm, delete it and run qterm again."
+				"Then import your old AddressBook by hand\n" 
+				"Sorry for any inconvenience. We try not to make this happen again.\n");
+		exit(-1);
+	}
+#endif
+
 	//install the translator
 	QString lang = conf->getItemValue("global","language");
 	if(lang !="eng" && !lang.isEmpty())
@@ -160,13 +303,45 @@ void iniSettings()
 	//set font
 	QString family = conf->getItemValue("global","font");
 	QString pointsize = conf->getItemValue("global","pointsize");
-	if( family!=NULL && pointsize.toInt()!=0 )
+	QString pixelsize = conf->getItemValue("global","pixelsize");
+	if( !family.isEmpty() )
 	{
-		QFont font(QString::fromLocal8Bit(family),pointsize.toInt(),QFont::Normal,false);
+		QFont font(QString::fromLocal8Bit(family));
+		if (pointsize.toInt() > 0)
+			font.setPointSize(pointsize.toInt());
+		if (pixelsize.toInt() > 0)
+			font.setPixelSize(pixelsize.toInt());
+		#if (QT_VERSION>=300)
+		QString bAA = conf->getItemValue("global", "antialias");
+		if(bAA!="0")
+			font.setStyleStrategy(QFont::PreferAntialias);
+		#endif
 		qApp->setFont(font,true);
 	}
 
+	// zmodem and pool directory
+	QString pathZmodem = QString::fromLocal8Bit(conf->getItemValue("preference", "zmodem"));
+	if(pathZmodem.isEmpty())
+		pathZmodem = pathCfg+"zmodem";
+	if(checkPath(pathZmodem)==-1)
+		return -1;
+	
+	QString pathPool = QString::fromLocal8Bit(conf->getItemValue("preference", "pool"));
+
+	if(pathPool.isEmpty()) 
+		pathPool = pathCfg+"pool/";
+
+	if (pathPool.right(1) != "/")
+		pathPool.append('/');
+
+	QString pathCache = pathPool+"shadow-cache/";
+	
+	if(checkPath(pathPool)==-1 || checkPath(pathCache) == -1)
+		return -1;
+
     delete conf;
+
+	return 0;
 }
 
 QStringList loadNameList( QTermConfig * pConf )
@@ -185,7 +360,7 @@ QStringList loadNameList( QTermConfig * pConf )
 	return listName;
 }
 
-void loadAddress( QTermConfig *pConf, int n, QTermParam& param )
+bool loadAddress( QTermConfig *pConf, int n, QTermParam& param )
 {
 	QString strTmp, strSection;
 	if(n<0)
@@ -193,6 +368,10 @@ void loadAddress( QTermConfig *pConf, int n, QTermParam& param )
 	else
 		strSection.sprintf("bbs %d", n);
 	
+	// check if larger than existence
+	strTmp = pConf->getItemValue("bbs list","num");
+	if(n>=strTmp.toInt())
+		return false;
 	param.m_strName = QString::fromLocal8Bit(pConf->getItemValue(strSection, "name"));
 	param.m_strAddr = pConf->getItemValue(strSection, "addr");
 	strTmp = pConf->getItemValue(strSection, "port");
@@ -221,6 +400,8 @@ void loadAddress( QTermConfig *pConf, int n, QTermParam& param )
 	param.m_nFontSize = strTmp.toInt();
 	param.m_clrFg.setNamedColor(pConf->getItemValue(strSection, "fgcolor"));
 	param.m_clrBg.setNamedColor(pConf->getItemValue(strSection, "bgcolor"));
+	param.m_strSchemaFile = pConf->getItemValue(strSection, "schemafile");
+	
 	param.m_strTerm = pConf->getItemValue(strSection, "termtype");
 	strTmp = pConf->getItemValue(strSection, "keytype");
 	param.m_nKey = strTmp.toInt();
@@ -250,6 +431,10 @@ void loadAddress( QTermConfig *pConf, int n, QTermParam& param )
 
 	strTmp = pConf->getItemValue(strSection, "maxidle");
 	param.m_nMaxIdle = strTmp.toInt();
+	param.m_strReplyKey = pConf->getItemValue(strSection, "replykey");
+	if(param.m_strReplyKey.isNull())
+		printf("loading null\n");
+			
 	param.m_strAntiString = pConf->getItemValue(strSection, "antiidlestring");
 	param.m_strAutoReply = QString::fromLocal8Bit(pConf->getItemValue(strSection, "autoreply"));
 	strTmp = pConf->getItemValue(strSection, "bautoreply");
@@ -261,9 +446,16 @@ void loadAddress( QTermConfig *pConf, int n, QTermParam& param )
 	param.m_nReconnectInterval = strTmp.toInt();
 	strTmp = pConf->getItemValue(strSection, "retrytimes");
 	param.m_nRetry = strTmp.toInt();
+
+	strTmp = pConf->getItemValue(strSection, "loadscript");
+	param.m_bLoadScript = (strTmp!="0");
+	param.m_strScriptFile = pConf->getItemValue(strSection, "scriptfile");
+	
 	strTmp = pConf->getItemValue(strSection, "menutype");
 	param.m_nMenuType = strTmp.toInt();
 	param.m_clrMenu.setNamedColor(pConf->getItemValue(strSection, "menucolor"));
+
+	return true;
 }
 
 void saveAddress(QTermConfig *pConf, int n, const QTermParam& param)
@@ -295,9 +487,11 @@ void saveAddress(QTermConfig *pConf, int n, const QTermParam& param)
 	pConf->setItemValue(strSection, "ansicolor", param.m_bAnsiColor?"1":"0");
 	pConf->setItemValue(strSection, "fontname", param.m_strFontName.local8Bit());
 	strTmp.setNum(param.m_nFontSize);
-	pConf->setItemValue(strSection, "fontsize",strTmp);
-	pConf->setItemValue(strSection, "fgcolor",param.m_clrFg.name());
+	pConf->setItemValue(strSection, "fontsize", strTmp);
+	pConf->setItemValue(strSection, "fgcolor", param.m_clrFg.name());
 	pConf->setItemValue(strSection, "bgcolor", param.m_clrBg.name());
+	pConf->setItemValue(strSection, "schemafile", param.m_strSchemaFile);
+	
 	pConf->setItemValue(strSection, "termtype",param.m_strTerm);
 	strTmp.setNum(param.m_nKey);
 	pConf->setItemValue(strSection, "keytype",strTmp);
@@ -326,6 +520,7 @@ void saveAddress(QTermConfig *pConf, int n, const QTermParam& param)
 
 	strTmp.setNum(param.m_nMaxIdle);
 	pConf->setItemValue(strSection, "maxidle", strTmp);
+	pConf->setItemValue(strSection, "replykey", param.m_strReplyKey);
 	pConf->setItemValue(strSection, "antiidlestring", param.m_strAntiString);
 	pConf->setItemValue(strSection, "bautoreply", param.m_bAutoReply?"1":"0");
 	pConf->setItemValue(strSection, "autoreply", param.m_strAutoReply.local8Bit());
@@ -335,12 +530,19 @@ void saveAddress(QTermConfig *pConf, int n, const QTermParam& param)
 	strTmp.setNum(param.m_nRetry);
 	pConf->setItemValue(strSection, "retrytimes", strTmp);
 
+	pConf->setItemValue(strSection, "loadscript", param.m_bLoadScript?"1":"0");
+	pConf->setItemValue(strSection, "scriptfile", param.m_strScriptFile);
+					
 	strTmp.setNum(param.m_nMenuType);
 	pConf->setItemValue(strSection, "menutype", strTmp);
 	pConf->setItemValue(strSection, "menucolor",param.m_clrMenu.name());
 
 }
 
+#ifdef HAVE_PYTHON
+PyThreadState * mainThreadState;
+#endif //HAVE_PYTHON
+
 int main( int argc, char ** argv ) 
 {
 
@@ -348,15 +550,45 @@ int main( int argc, char ** argv )
 
 	qApp=&a;
 
-#ifndef _OS_WIN32_
     if( iniWorkingDir( argv[0] )<0 )
     {
 	    return -1;
     }
-#endif
-      //set font
-    iniSettings();
+    //set font
+    if( iniSettings()<0 )
+	{
+		return -1;
+	}
 
+#ifdef HAVE_PYTHON
+	  // initialize Python
+    Py_Initialize();
+    // initialize thread support
+    PyEval_InitThreads();
+    mainThreadState = NULL;
+    // save a pointer to the main PyThreadState object
+    mainThreadState = PyThreadState_Get();
+
+	
+	// add path
+	PyRun_SimpleString("import sys\n");
+	QString pathCmd;
+	// pathLib/script
+	pathCmd = "sys.path.insert(0,'";
+	pathCmd += pathLib+"script')";
+	PyRun_SimpleString(strdup(pathCmd));
+
+#if !defined(_OS_WIN32_) && !defined(Q_OS_WIN32)
+	// $HOME/.qterm/script, override 
+	pathCmd = "sys.path.insert(0,'";
+	pathCmd += pathCfg+"script')";
+	PyRun_SimpleString(strdup(pathCmd));
+#endif	
+	// release the lock
+    PyEval_ReleaseLock();
+#endif // HAVE_PYTHON
+
+  
     QTermFrame * mw = new QTermFrame();
     mw->setCaption( "QTerm "+QString(VERSION) );
 	mw->setIcon( QPixmap(pathLib+"pic/qterm_32x32.png") );
@@ -364,5 +596,15 @@ int main( int argc, char ** argv )
     mw->show();
     a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
     int res = a.exec();
+
+#ifdef HAVE_PYTHON
+	// shut down the interpreter
+	PyInterpreterState * mainInterpreterState = mainThreadState->interp;
+	// create a thread state object for this thread
+	PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
+	PyThreadState_Swap(myThreadState);
+	PyEval_AcquireLock();
+	Py_Finalize();
+#endif // HAVE_PYTHON
     return res;
 }
