@@ -25,18 +25,18 @@ QTermBuffer::QTermBuffer( int line, int column, int limit )
 {
 	m_lin = line;
 	m_col = column;
-	m_limit = limit+line;
+	m_limit = limit;
 	
 	m_lines = 0;
-	m_lineList.setAutoDelete(true);
 
 	while(  m_lineList.count()< m_lin )
 		 m_lineList.append( new QTermTextLine );
 
-	m_lineList.setAutoDelete( true );
+	m_lineList.setAutoDelete(true);
 
 	m_curAttr = SETCOLOR( NO_COLOR ) | SETATTR( NO_ATTR );
 
+
 	m_caretX = 0;	
 	m_caretY = 0;
 
@@ -58,27 +58,33 @@ QTermBuffer::QTermBuffer( int line, int column, int limit )
 
 QTermBuffer::~QTermBuffer()
 {
+
 }
 
 void QTermBuffer::setSize(int col, int lin)
 {
+	if(m_col==col && m_lin==lin)
+		return;
+
 	if(m_lin<lin)
 		for(int i=0; i<lin-m_lin; i++ )
 			m_lineList.append( new QTermTextLine );
-	else
+	else if(m_lin>lin)
 		for(int i=0; i<m_lin-lin; i++ )
-			m_lineList.removeLast();
+			m_lineList.remove(m_lines+m_top);
 	
 	m_col = col;
 	m_lin = lin;
-
+	
 	m_top = 0;
 	m_bottom = m_lin -1;
 
-	m_ptSelStart = QPoint(-1,-1);
-	m_ptSelEnd = QPoint(-1,-1);
-	
+	m_caretY = QMIN(m_caretY, lin-1);
+	m_oldCaretY = QMIN(m_caretY, lin-1);
 
+	clearSelect();
+
+	emit windowSizeChanged(m_col,m_lin);
 
 	emit bufferSizeChanged();
 }
@@ -115,7 +121,13 @@ void QTermBuffer::setCurAttr( short attr )
 void QTermBuffer::setBuffer( const QCString& cstr, int n )
 {
 	
-	QTermTextLine * line = line = m_lineList.at( m_lines + m_caretY );
+	QTermTextLine * line =  m_lineList.at( m_lines + m_caretY );
+	
+	if(line==NULL)
+	{	
+		qWarning("setBuffer null line");
+		return;
+	}
 
 	if ( Insert_Mode /*bInsert*/ )
 		line->insertText( cstr, m_curAttr, m_caretX );
@@ -150,12 +162,11 @@ void QTermBuffer::newLine()
 void QTermBuffer::tab()
 {
 	int x = ( m_caretX + 8 ) & -8;	// or ( caretX/8 + 1 ) * 8 
-
 	moveCursor( x, m_caretY );
 
-//	QCString cstr;
-//	cstr.fill(' ',x-m_caretX);
-//	setBuffer(cstr,x-m_caretX);
+	QCString cstr;
+	cstr.fill(' ',x-m_caretX);
+	setBuffer(cstr,x-m_caretX);
 }
 
 void QTermBuffer::setMargins( int top, int bottom )
@@ -169,7 +180,7 @@ void QTermBuffer::moveCursor( int x, int y )
 {
 	// detect index boundary	
 	if ( x >= m_col )
-		x = m_col - 1;
+		x = m_col;
 	if ( x < 0 )
 		x = 0;
 
@@ -420,7 +431,7 @@ void QTermBuffer::addHistoryLine( int n )
 {
 	while( n )
 	{
-		if( m_lines == m_limit-m_lin )
+		if( m_lines == m_limit )
 		{
 			m_lineList.remove(uint(0));
 			//m_ptSelStart.setY( m_ptSelStart.y()-1 );
@@ -430,7 +441,7 @@ void QTermBuffer::addHistoryLine( int n )
 		}
 		
 		m_lineList.append( new QTermTextLine );
-		m_lines = QMIN(m_lines+1,m_limit-m_lin);
+		m_lines = QMIN(m_lines+1,m_limit);
 		n--;
 	}
 	
@@ -609,12 +620,16 @@ QCString QTermBuffer::getSelectText( bool rect, bool color, const QCString& esca
 		int pos=cstrTemp.findRev(QRegExp("[^\\s]"));
 		#endif
 		cstrTemp.truncate(pos+1);
-	
 		cstrSelect += cstrTemp;
-		#ifdef _OS_WIN32_
-		cstrSelect += '\r';
-		#endif
-		cstrSelect += '\n';
+		
+		// add newline except the last line
+		if(i != m_ptSelEnd.y())
+		{
+			#if defined(_OS_WIN32_) || defined(Q_OS_WIN32)
+			cstrSelect += '\r';
+			#endif
+			cstrSelect += '\n';
+		}
 	}
 
 	return cstrSelect;
@@ -636,3 +651,4 @@ QRect QTermBuffer::getSelectRect( int index, bool rect )
 				else
 					return QRect( 0, index, m_col, 1);
 }
+#include <qtermbuffer.moc>
