@@ -19,6 +19,8 @@ AUTHOR:        kingson fiasco
 #include <stdio.h>
 #include <qcstring.h>
 
+#define MODE_MouseX11	0
+
 /************************************************************************/
 // state for FSM
 // please read ANSI decoding
@@ -30,8 +32,8 @@ StateOption QTermDecode::normalState[] =
     { CHAR_TAB, 	&QTermDecode::tab,			normalState },
     { CHAR_BS,  	&QTermDecode::bs,			normalState },
     { CHAR_BELL,  	&QTermDecode::bell,			normalState },
-    { CHAR_ESC, 	0,					escState    },
-    { CHAR_NORMAL, 	&QTermDecode::normalInput,  		normalState }
+    { CHAR_ESC, 	0,							escState    },
+    { CHAR_NORMAL, 	&QTermDecode::normalInput, 	normalState }
 };
 
 // state after a ESC_CHAR
@@ -39,10 +41,20 @@ StateOption QTermDecode::normalState[] =
 StateOption QTermDecode::escState[] =
 {
     { '[', 		&QTermDecode::clearParam,  		bracketState },
-    { CHAR_NORMAL, 	0,					normalState  }
+	// VT52
+	{ 'A', 		&QTermDecode::cursorUp,			normalState	},
+    { 'B',		&QTermDecode::cursorDown,		normalState	},
+    { 'C', 		&QTermDecode::cursorRight,		normalState },
+    { 'D', 		&QTermDecode::cursorLeft,		normalState },
+    { 'J', 		&QTermDecode::eraseScreen,		normalState },
+    { 'K', 		&QTermDecode::eraseLine,		normalState },
+	{ 'Z',		&QTermDecode::test,				normalState	},
+	{ '>',		&QTermDecode::test,				normalState	},
+	{ '<',		&QTermDecode::test,				normalState	},
+    { CHAR_NORMAL, 	0,							normalState	}
 };
 
-// state after '*['
+// state after ESC [
 StateOption QTermDecode::bracketState[] =
 {
     { '0', 		&QTermDecode::paramDigit,		bracketState },
@@ -57,24 +69,26 @@ StateOption QTermDecode::bracketState[] =
     { '9', 		&QTermDecode::paramDigit,		bracketState },
     { ';', 		&QTermDecode::nextParam,		bracketState },
 
+	{ '?',		&QTermDecode::clearParam,		privateState},
+	
     { 'A', 		&QTermDecode::cursorUp,			normalState },
     { 'B', 		&QTermDecode::cursorDown,		normalState },
     { 'C', 		&QTermDecode::cursorRight,		normalState },
     { 'D', 		&QTermDecode::cursorLeft,		normalState },
-    { 'H', 		&QTermDecode::cursorPosition,  		normalState },
+    { 'H', 		&QTermDecode::cursorPosition,  	normalState },
     { 'J', 		&QTermDecode::eraseScreen,		normalState },
     { 'K', 		&QTermDecode::eraseLine,		normalState },
     { 'L', 		&QTermDecode::insertLine,		normalState },
     { 'M', 		&QTermDecode::deleteLine,		normalState },
     { 'P', 		&QTermDecode::deleteStr,		normalState },
     { 'X', 		&QTermDecode::eraseStr,			normalState },
-    { 'f', 		&QTermDecode::cursorPosition,  		normalState },
+    { 'f', 		&QTermDecode::cursorPosition,  	normalState },
     { 'h',		&QTermDecode::setMode,			normalState },
     { 'l',		&QTermDecode::resetMode,		normalState },
     { 'm', 		&QTermDecode::getAttr,			normalState },
     { 'r',		&QTermDecode::setMargins,		normalState },
     { 's', 		&QTermDecode::saveCursor,		normalState },
-    { 'u', 		&QTermDecode::restoreCursor,		normalState },
+    { 'u', 		&QTermDecode::restoreCursor,	normalState },
     { '@', 		&QTermDecode::insertStr,		normalState },
 
     { CHAR_CR, 		&QTermDecode::cr,			bracketState },
@@ -83,7 +97,31 @@ StateOption QTermDecode::bracketState[] =
     { CHAR_TAB,  	&QTermDecode::tab,			bracketState },
     { CHAR_BS, 	 	&QTermDecode::bs,			bracketState },
     { CHAR_BELL, 	&QTermDecode::bell,			bracketState },
-    { CHAR_NORMAL, 	0,					normalState }
+    { CHAR_NORMAL, 	0,							normalState }
+};
+
+// state after ESC [ ?
+
+StateOption QTermDecode::privateState[] =
+{
+	{ '0', 		&QTermDecode::paramDigit,			privateState },
+    { '1', 		&QTermDecode::paramDigit,			privateState },
+    { '2', 		&QTermDecode::paramDigit,			privateState },
+    { '3', 		&QTermDecode::paramDigit,			privateState },
+    { '4', 		&QTermDecode::paramDigit,			privateState },
+    { '5',		&QTermDecode::paramDigit,			privateState },
+    { '6', 		&QTermDecode::paramDigit,			privateState },
+    { '7', 		&QTermDecode::paramDigit,			privateState },
+    { '8', 		&QTermDecode::paramDigit,			privateState },
+    { '9', 		&QTermDecode::paramDigit,			privateState },
+    { ';', 		&QTermDecode::nextParam,			privateState },
+
+	{ 'h',		&QTermDecode::setMode,				normalState	 },
+	{ 'l',		&QTermDecode::resetMode,			normalState	 },
+	{ 's',		&QTermDecode::saveMode,				normalState	 },
+	{ 'r',		&QTermDecode::restoreMode,			normalState	 },
+
+	{ CHAR_NORMAL, 	0,								normalState  }
 };
 
 QTermDecode::QTermDecode( QTermBuffer * buffer )
@@ -97,6 +135,8 @@ QTermDecode::QTermDecode( QTermBuffer * buffer )
 	m_curAttr = m_defAttr;
 
 	m_pBuffer->setCurAttr( m_curAttr );
+
+	bCurMode[MODE_MouseX11]=bSaveMode[MODE_MouseX11]=false;
 }
 
 QTermDecode::~QTermDecode()
@@ -127,7 +167,8 @@ void QTermDecode::decode( const char *cstr, int length )
 		// current state always be initialized to point to the deginning of three structures
 		// ( normalState, escState, bracketState )
 		i = 0;
-		while ( currentState[i].byte != CHAR_NORMAL && currentState[i].byte != inputData[dataIndex] )
+		while ( currentState[i].byte != CHAR_NORMAL 
+			&& currentState[i].byte != inputData[dataIndex] )
 			   i++;
 
 		// action must be allowed to redirect state change
@@ -450,40 +491,89 @@ void QTermDecode::getAttr()
 
 void QTermDecode::setMode()
 {
-	int n = param[0];
-
-	switch(n)
+	for( int i=0; i<=nParam; i++)
 	{
-		case 4:
-			m_pBuffer->setMode( INSERT_MODE );
-			break;
-		case 20:
-			m_pBuffer->setMode( INSERT_MODE );
-			break;
-		default:
-			break;
+		int n = param[i];
+		switch(n)
+		{
+			case 4:
+				m_pBuffer->setMode( INSERT_MODE );
+				break;
+			case 20:
+				m_pBuffer->setMode( INSERT_MODE );
+				break;
+			case 1000:
+			case 1001:
+				emit mouseMode(true);
+				bCurMode[MODE_MouseX11]=true;
+				break;
+			default:
+				break;
+		}
 	}
 }
 
 void QTermDecode::resetMode()
 {
-	int n = param[0];
-
-	switch(n)
+	for( int i=0; i<=nParam; i++)
 	{
-		case 4:
-			m_pBuffer->resetMode( INSERT_MODE );
-			break;
-		case 20:
-			m_pBuffer->resetMode( NEWLINE_MODE );
-			break;
-		default:
+		int n = param[i];
+		switch(n)
+		{
+			case 4:
+				m_pBuffer->resetMode( INSERT_MODE );
+				break;
+			case 20:
+				m_pBuffer->resetMode( NEWLINE_MODE );
+				break;
+			case 1000:
+			case 1001:
+				bCurMode[MODE_MouseX11]=false;
+				emit mouseMode(false);
+				break;
+			default:
 			break;
+		}
 	}
-
 }
 
+void QTermDecode::saveMode()
+{
+	for( int i=0; i<=nParam; i++)
+	{
+		int n = param[i];
+		switch(n)
+		{
+			case 1000:
+			case 1001:
+				bSaveMode[MODE_MouseX11]=bCurMode[MODE_MouseX11];
+				break;
+			default:
+			break;
+		}
+	}
+}
+
+void QTermDecode::restoreMode()
+{
+	for( int i=0; i<=nParam; i++)
+	{
+		int n = param[i];
+		switch(n)
+		{
+			case 1000:
+			case 1001:
+				bCurMode[MODE_MouseX11]=bSaveMode[MODE_MouseX11];
+				emit mouseMode( bCurMode[MODE_MouseX11] );
+				break;
+			default:
+			break;
+		}
+	}
+}
+
+
 void QTermDecode::test()
 {
-		printf("QTermDecode::test()\n");
 }
+#include <qtermdecode.moc>
