@@ -8,9 +8,10 @@ AUTHOR:        kingson fiasco
                                     NOTE
  This file may be used, distributed and modified without limitation.
  *******************************************************************************/
+
+#include "qtermwindow.h"
 #include "qtermframe.h"
 #include "qtermwndmgr.h"
-#include "qtermwindow.h"
 #include "qtermtimelabel.h"
 #include "qtermconfig.h"
 #include "qterm.h"
@@ -22,8 +23,11 @@ AUTHOR:        kingson fiasco
 #include "prefdialog.h"
 #include "quickdialog.h"
 #include "keydialog.h"
+#include "trayicon.h"
+#include "imageviewer.h"
+#include "qtermimage.h"
 
-#ifndef _OS_WIN32_
+#if !defined(_OS_WIN32_) && !defined(Q_OS_WIN32)
 #include <unistd.h>
 #endif
 
@@ -44,13 +48,19 @@ AUTHOR:        kingson fiasco
 #include <qstatusbar.h>
 #include <qmessagebox.h>
 #include <qhbox.h>
-/*
+#include <qregexp.h>
+
+#if QT_VERSION < 300
 #include <qplatinumstyle.h>
 #include <qmotifstyle.h>
 #include <qmotifplusstyle.h>
 #include <qcdestyle.h>
 #include <qsgistyle.h>
-*/
+#else
+#include <qstylefactory.h>
+#include <qstyle.h>
+#endif
+
 #include <qfont.h>
 #include <qfontdialog.h>
 #include <qtranslator.h>
@@ -67,27 +77,35 @@ AUTHOR:        kingson fiasco
 #include <qaction.h>
 #include <qinputdialog.h>
 
-extern char fileCfg[];
-extern char addrCfg[];
+extern QString fileCfg;
+extern QString addrCfg;
 
 extern QString pathLib;
 extern QString pathPic;
+extern QString pathCfg;
 
+extern void clearDir(const QString& );
 extern QStringList loadNameList(QTermConfig *);
-extern void loadAddress(QTermConfig *, int, QTermParam &);
+extern bool loadAddress(QTermConfig *, int, QTermParam &);
 extern void saveAddress(QTermConfig *, int, const QTermParam &);
 
+
+
 //constructor
 QTermFrame::QTermFrame()
     : QMainWindow( 0, "QTerm", WDestructiveClose )
 {
-
 //set the layout
 	QVBox *vb = new QVBox( this );
 	vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
 	setCentralWidget( vb );
 	ws = new QWorkspace( vb );
 
+	tray = 0;
+	trayMenu = 0;
+	
+	viewer = NULL;
+	
 //set menubar
 	addMainMenu();
 
@@ -113,23 +131,33 @@ QTermFrame::QTermFrame()
 	tabBar=new QTabBar(hb);
 	connect(tabBar,SIGNAL(selected(int)),this,SLOT(selectionChanged(int)));
 	tabBar->setShape(QTabBar::TriangularBelow);
-//create a label to display current time
-	labelTime=new QTermTimeLabel(statusBar());
-	statusBar()->addWidget(labelTime,10,TRUE);
-	/*
-//create the float input
-	dock = new QToolBar(this);
-	dock->setOrientation(Qt::Horizontal);
-	dock->setHorizontallyStretchable(true);	
-	input = new QLineEdit(dock);
-	connect(input, SIGNAL(returnPressed()), this, SLOT(dialogInput()));
-	dock->setWidget(input);
-	moveDockWindow(dock, Bottom);
-	*/
+	//tabBar->setShape(QTabBar::RoundedBelow);
+
+//create a progress bar to notify the download process
+	m_pStatusBar = new QTerm::StatusBar(statusBar(), "mainStatusBar");
+	statusBar()->addWidget(m_pStatusBar, 0, false);
+	connect(m_pStatusBar, SIGNAL(started()), this, SLOT(startDown()));
+	connect(m_pStatusBar, SIGNAL(ended()), this, SLOT(endDown()));
+
 
 //create the window manager to deal with the window-tab-icon pairs
 	wndmgr=new QTermWndMgr(this);
 
+// expressly connect sites in addressbook
+	QAccel *accel2 = new QAccel(this);
+	accel2->insertItem(CTRL+ALT+Key_1, 0);
+	accel2->insertItem(CTRL+ALT+Key_2, 1);
+	accel2->insertItem(CTRL+ALT+Key_3, 2);
+	accel2->insertItem(CTRL+ALT+Key_4, 3);
+	accel2->insertItem(CTRL+ALT+Key_5, 4);
+	accel2->insertItem(CTRL+ALT+Key_6, 5);
+	accel2->insertItem(CTRL+ALT+Key_7, 6);
+	accel2->insertItem(CTRL+ALT+Key_8, 7);
+	accel2->insertItem(CTRL+ALT+Key_9, 8);
+	accel2->insertItem(CTRL+ALT+Key_9, 9);
+	connect( accel2, SIGNAL(activated(int)), 
+	this,SLOT(connectMenuActivated(int)));
+
 // ALT+# to switch between 10 windows
 	QAccel *accel = new QAccel(this);
 	accel->insertItem(ALT+Key_1, 1);
@@ -152,6 +180,7 @@ QTermFrame::QTermFrame()
 	//initialize all settings
 	iniSetting();
 
+	installEventFilter(this);
 }
 
 //destructor
@@ -183,7 +212,7 @@ void QTermFrame::iniSetting()
 			showMaximized();
 		else
 		{
-			char * size = conf->getItemValue("global","size");
+			const char * size = conf->getItemValue("global","size");
 			if( size!=NULL )
 			{
 				int x,y,cx,cy;
@@ -194,45 +223,53 @@ void QTermFrame::iniSetting()
 		}
 	}
 
-	/* FIXME qt3.1.1 on debian/sid doesnot support style
-	strTmp = conf->getItemValue("global","theme");
-	theme = strTmp.toInt();
-	switch( theme )
+	/* FIXME qt3.1.1 on debian/sid doesnot support style */
+	theme = conf->getItemValue("global","theme");
+#if QT_VERSION < 300
+	if(theme.isEmpty())
 	{
-		case 0:
-			break;
-		case 1:
-			#ifndef QT_NO_STYLE_CDE
-			qApp->setStyle( new QCDEStyle(true) );
-			#endif
-			break;
-		case 2:
-			#ifndef QT_NO_STYLE_MOTIF
-			qApp->setStyle( new QMotifStyle(true) );
-			#endif
-			break;
-		case 3:
-			#ifndef QT_NO_STYLE_MOTIFPLUS
-			qApp->setStyle( new QMotifPlusStyle(true) );
-			#endif
-			break;
-		case 4:
-			#ifndef QT_NO_STYLE_PLATINUM
-			qApp->setStyle( new QPlatinumStyle() );
-			#endif
-			break;
-		case 5:
-			#ifndef QT_NO_STYLE_SGI
-			qApp->setStyle( new QSGIStyle(true) );
-			#endif
-			break;
-		case 6:
-			#ifndef QT_NO_STYLE_WINDOWS
-			qApp->setStyle( new QWindowsStyle() );
-			#endif
-			break;
 	}
-	*/
+	else if(theme=="CDE")
+	{
+		#ifndef QT_NO_STYLE_CDE
+		qApp->setStyle( new QCDEStyle(true) );
+		#endif
+	}
+	else if(theme=="Motif")
+	{
+		#ifndef QT_NO_STYLE_MOTIF
+		qApp->setStyle( new QMotifStyle(true) );
+		#endif
+	}
+	else if(theme=="MotifPlus")
+	{
+		#ifndef QT_NO_STYLE_MOTIFPLUS
+		qApp->setStyle( new QMotifPlusStyle(true) );
+		#endif
+	}
+	else if(theme=="Platinum")
+	{
+		#ifndef QT_NO_STYLE_PLATINUM
+		qApp->setStyle( new QPlatinumStyle() );
+		#endif
+	}
+	else if(theme=="SGI")
+	{
+		#ifndef QT_NO_STYLE_SGI
+		qApp->setStyle( new QSGIStyle(true) );
+		#endif
+	}
+	else if(theme == "Windows")
+	{
+		#ifndef QT_NO_STYLE_WINDOWS
+		qApp->setStyle( new QWindowsStyle() );
+		#endif
+	}
+#else
+	QStyle * style = QStyleFactory::create(theme);
+	if (style)
+		qApp->setStyle(style);
+#endif
 	
 	//language
 	strTmp = conf->getItemValue("global","language");
@@ -294,6 +331,8 @@ void QTermFrame::iniSetting()
 
 	loadPref( conf );
 
+	setUseDock(m_pref.bTray);
+
 	delete conf;
 }
 
@@ -323,7 +362,25 @@ void QTermFrame::loadPref( QTermConfig * conf )
 	m_pref.strHttp = strTmp;
 	strTmp = conf->getItemValue("preference","antialias");
 	m_pref.bAA=(strTmp!="0");
+	strTmp = conf->getItemValue("preference","tray");
+	m_pref.bTray=(strTmp!="0");
+	strTmp = conf->getItemValue("preference","playmethod");
+	m_pref.nMethod=strTmp.toInt();
+	strTmp = conf->getItemValue("preference","externalplayer");
+	m_pref.strPlayer=strTmp;
 
+	strTmp = conf->getItemValue("preference","clearpool");
+	m_pref.bClearPool=(strTmp!="0");
+	strTmp = conf->getItemValue("preference","pool");
+	m_pref.strPoolPath=strTmp.isEmpty()?pathCfg+"pool/":strTmp;
+	if( m_pref.strPoolPath.right(1) != "/" )
+		m_pref.strPoolPath.append('/');
+	strTmp = conf->getItemValue("preference","zmodem");
+	m_pref.strZmPath=strTmp.isEmpty()?pathCfg+"zmodem/":strTmp;
+	if( m_pref.strZmPath.right(1) != "/" )
+		m_pref.strZmPath.append('/');
+	strTmp = conf->getItemValue("preference","image");
+	m_pref.strImageViewer=QString::fromLocal8Bit(strTmp);
 }
 
 //save current setting to qterm.cfg
@@ -334,8 +391,11 @@ void QTermFrame::saveSetting()
 	QCString cstrTmp;
 	//save font
 	conf->setItemValue("global","font",qApp->font().family().local8Bit());
-	cstrTmp.setNum(qApp->font().pointSize());
+	cstrTmp.setNum(QFontInfo(qApp->font()).pointSize());
 	conf->setItemValue("global","pointsize",cstrTmp);
+	cstrTmp.setNum(QFontInfo(qApp->font()).pixelSize());
+	conf->setItemValue("global","pixelsize",cstrTmp);
+
 	//save window position and size
 	if(isMaximized())
 	{
@@ -353,24 +413,24 @@ void QTermFrame::saveSetting()
 	else
 		conf->setItemValue("global","fullscreen","0");
 	
-	cstrTmp.setNum(theme);
-	conf->setItemValue("global","theme",cstrTmp);
+	// cstrTmp.setNum(theme);
+	conf->setItemValue("global","theme",theme);
 
 	int hide,dock,index,nl,extra;
 
 	cstrTmp = conf->getItemValue("global", "bbsbar");
 	sscanf(cstrTmp,"%d %d %d %d %d",&hide,&dock,&index,&nl,&extra);
-	conf->setItemValue("global","bbsbar", valueToString(mdiconnectTools->isVisible(), 
+	conf->setItemValue("global","bbsbar", valueToString(mdiconnectTools->isVisibleTo(this),
 							(int)dock, index, nl==1?true:false, extra));
 
 	cstrTmp = conf->getItemValue("global", "keybar");
 	sscanf(cstrTmp,"%d %d %d %d %d",&hide,&dock,&index,&nl,&extra);
-	conf->setItemValue("global","keybar", valueToString(key->isVisible(), 
+	conf->setItemValue("global","keybar", valueToString(key->isVisibleTo(this), 
 							(int)dock, index, nl==1?true:false, extra));
 
 	cstrTmp = conf->getItemValue("global", "mainbar");
 	sscanf(cstrTmp,"%d %d %d %d %d",&hide,&dock,&index,&nl,&extra);
-	conf->setItemValue("global","mainbar",valueToString(mdiTools->isVisible(),
+	conf->setItemValue("global","mainbar",valueToString(mdiTools->isVisibleTo(this),
 							(int)dock, index, nl==1?true:false, extra) );
 
 	
@@ -387,18 +447,6 @@ void QTermFrame::saveSetting()
 	delete conf;
 }
 
-void QTermFrame::dialogInput()
-{
-	QCString cstr;
-	if(m_pref.nXIM==0)
-		cstr = U2G(input->text());
-	else	
-		cstr = U2B(input->text());
-		
-	wndmgr->activeWindow()->externInput(cstr);
-	input->clear();
-}
-
 //addressbook
 void QTermFrame::addressBook()
 {
@@ -424,8 +472,33 @@ void QTermFrame::quickLogin()
 	}
 }
 
+void QTermFrame::exitQTerm()
+{
+	while( wndmgr->count()>0 ) 
+	{
+		bool closed = ws->activeWindow()->close();
+		if(!closed)
+		{
+			return;
+		}
+	}
+
+	saveSetting();
+	// clear zmodem and pool if needed
+	if(m_pref.bClearPool)
+	{
+		clearDir(m_pref.strZmPath);
+		clearDir(m_pref.strPoolPath);
+		clearDir(m_pref.strPoolPath+"shadow-cache/");
+	}
+
+	setUseDock(false);
+	qApp->quit();
+}
+
 //create a new display window
-QTermWindow * QTermFrame::newWindow( const QTermParam&  param, int index )
+//QTermWindow * QTermFrame::newWindow( const QTermParam&  param, int index )
+void QTermFrame::newWindow( const QTermParam&  param, int index )
 {
 	QTermWindow * window=new QTermWindow( this, param, index, ws,
 					0,  WDestructiveClose );
@@ -443,14 +516,20 @@ QTermWindow * QTermFrame::newWindow( const QTermParam&  param, int index )
 	
 	//add window-tab-icon to window manager
 	wndmgr->addWindow(window,qtab,icon);
-
-	window->showMaximized();
+	
+	if( ws->windowList().isEmpty() )
+		window->showMaximized();
+	else
+	{
+		ws->setFocus();
+		window->show();
+	}
 
 	//activte the window-tab
-	window->setFocus();
-	wndmgr->activateTheTab(window);
+//	window->setFocus();
+//	wndmgr->activateTheTab(window);
 
-	return window;
+//	return window;
 }
 
 //the tabbar selection changed
@@ -463,15 +542,22 @@ void QTermFrame::selectionChanged(int n)
 void QTermFrame::aboutQTerm()
 {
 	aboutDialog about(this);
-
+	
 	about.exec();
 }
 
 //slot Help->Homepage
 void QTermFrame::homepage()
 {
-	QCString cstrCmd = m_pref.strHttp.local8Bit()+" http://qterm.gnuchina.org";
-	#ifndef _OS_WIN32_
+    QCString cstrCmd = m_pref.strHttp.local8Bit();
+	QCString cstrUrl = "http://qterm.sourceforge.net";
+
+    if(cstrCmd.find("%L")==-1)
+        cstrCmd += " \"" + cstrUrl +"\"";
+    else
+        cstrCmd.replace(QRegExp("%L",false), cstrUrl);
+
+	#if !defined(_OS_WIN32_) && !defined(Q_OS_WIN32)
 	cstrCmd += " &";
 	#endif
 	system(cstrCmd);
@@ -488,6 +574,13 @@ void QTermFrame::windowsMenuAboutToShow()
 		windowsMenu->setItemEnabled( tileId, FALSE );
 	}
 	windowsMenu->insertSeparator();
+
+#ifdef Q_OS_MACX
+	// used to dock the programe
+	if(isHidden())
+		windowsMenu->insertItem(tr("Main Window"), this, SLOT(trayShow()));
+#endif
+
 	QWidgetList windows = ws->windowList();
 	for ( int i = 0; i < int(windows.count()); ++i ) 
 	{
@@ -505,8 +598,7 @@ void QTermFrame::windowsMenuActivated( int id )
 	if ( w ) 
 	{
 		w->showNormal();
-		w->setFocus();
-		wndmgr->activateTheTab((QTermWindow*)w);
+//		wndmgr->activateTheTab((QTermWindow*)w);
 	} 
 }
 
@@ -540,9 +632,8 @@ void QTermFrame::connectMenuActivated( int id )
 {
 	QTermConfig *pConf = new QTermConfig(addrCfg);
 	QTermParam param;
-	loadAddress(pConf, connectMenu->itemParameter(id), param);
-	newWindow(param, connectMenu->itemParameter(id));
-	
+	if(loadAddress(pConf, connectMenu->itemParameter(id), param))
+		newWindow(param, connectMenu->itemParameter(id));
 	delete pConf;
 }
 
@@ -557,17 +648,36 @@ void QTermFrame::switchWin(int id)
 		wndmgr->activeNextPrev(false);
 		return;
 	}
-	if(id==201)
+	if(id==201 || id==202)
 	{
 		wndmgr->activeNextPrev(true);
 		return;
 	}
 
 	QWidget *w = windows.at(id-1);
+	if(w == ws->activeWindow() )
+		return;
+
 	if(w!=NULL)
-		w->setFocus();
+		w->showNormal();
 }
 
+bool QTermFrame::eventFilter(QObject *o, QEvent *e)
+{
+/*
+	if( o==this && m_pref.bTray)
+	{
+		if( e->type()==QEvent::ShowMinimized && m_pref.bTray )
+		{
+			printf("QTermFrame::eventFilter\n");
+			trayHide();
+			return true;
+		}
+	}
+	return false;
+*/
+	return false;
+}
 
 //slot draw something e.g. logo in the background
 //TODO : draw a pixmap in the background
@@ -578,17 +688,41 @@ void QTermFrame::paintEvent( QPaintEvent * )
 
 void QTermFrame::closeEvent(QCloseEvent * clse)
 {
-	while( wndmgr->count()>0 ) 
-	{
-		bool closed = ws->activeWindow()->close();
-		if(!closed)
-		{
-			clse->ignore();
-			return;
-		}
-	}
-	saveSetting();
-	clse->accept();
+        QWidgetList windows = ws->windowList();
+        for ( int i = 0; i < int(windows.count()); ++i )
+        {
+
+                if( ( (QTermWindow *)windows.at(i) )->isConnected() )
+                {
+                        if (m_pref.bTray) {
+                                trayHide();
+                                return;
+                        }
+                }
+        }
+        while( wndmgr->count()>0 )
+        {
+                bool closed = ws->activeWindow()->close();
+                if(!closed)
+                {
+                        return;
+                }
+        }
+
+        saveSetting();
+        // clear zmodem and pool if needed
+        if(m_pref.bClearPool)
+        {
+                clearDir(m_pref.strZmPath);
+                clearDir(m_pref.strPoolPath);
+		clearDir(m_pref.strPoolPath+"shadow-cache/");
+
+        }
+
+        setUseDock(false);
+
+        clse->accept();
+
 }
 
 void QTermFrame::langEnglish()
@@ -772,6 +906,12 @@ void QTermFrame::uiFont()
 {
 	bool ok;
 	QFont font = QFontDialog::getFont(&ok,qApp->font());
+	
+	#if (QT_VERSION>=300)
+	if(m_pref.bAA)
+		font.setStyleStrategy(QFont::PreferAntialias);
+	#endif
+
 	if(ok==true)
 	{
 		qApp->setFont(font,TRUE);
@@ -783,9 +923,27 @@ void QTermFrame::fullscreen()
 	m_bFullScreen = ! m_bFullScreen;
 	
 	if( m_bFullScreen )
+	{
+		menuBar()->hide();
+		mdiTools->hide();
+		mdiconnectTools->hide();
+		key->hide();
+		hideScroll();
+		showStatusBar();
+		showSwitchBar();
 		showFullScreen();
+	}
 	else
+	{
+		menuBar()->show();
+		mdiTools->show();
+		mdiconnectTools->show();
+		key->show();
+		emit updateScroll();
+		showStatusBar();
+		showSwitchBar();
 		showNormal();
+	}
 
 	menuBar()->setItemChecked( ID_VIEW_FULL, m_bFullScreen );
 
@@ -803,70 +961,85 @@ void QTermFrame::bosscolor()
 void QTermFrame::themesMenuAboutToShow()
 {
 	themesMenu->clear();
-/*	
-	insertThemeItem(0);	
+#if QT_VERSION < 300
+	insertThemeItem("default");	
 	#ifndef QT_NO_STYLE_CDE
-	insertThemeItem(1);
+	insertThemeItem("CDE");
 	#endif
 	#ifndef QT_NO_STYLE_MOTIF
-	insertThemeItem(2);
+	insertThemeItem("Motif");
 	#endif
 	#ifndef QT_NO_STYLE_MOTIFPLUS
-	insertThemeItem(3);
+	insertThemeItem("MotifPlus");
 	#endif
 	#ifndef QT_NO_STYLE_PLATINUM
-	insertThemeItem(4);
+	insertThemeItem("Platinum");
 	#endif
 	#ifndef QT_NO_STYLE_SGI
-	insertThemeItem(5);
+	insertThemeItem("SGI");
 	#endif
 	#ifndef QT_NO_STYLE_WINDOWS
-	insertThemeItem(6);
+	insertThemeItem("Windows");
 	#endif
-*/
+#else
+	QStringList styles = QStyleFactory::keys();
+	for(QStringList::ConstIterator it=styles.begin(); it!=styles.end(); it++)
+		insertThemeItem(*it);
+#endif
 }
 
 void QTermFrame::themesMenuActivated( int id )
 {
 
-	theme = themesMenu->itemParameter(id);
-	/*  FIXME qt3.1.1 on debian/sid doesnot support style
-	switch( theme )
+	theme = themesMenu->text(id);
+	fprintf(stderr, "Theme: %s\n", theme.ascii());
+
+#if QT_VERSION < 300
+	if(theme=="default")
 	{
-		case 0:
-			break;
-		case 1:
-			#ifndef QT_NO_STYLE_CDE
-			qApp->setStyle( new QCDEStyle(true) );
-			#endif
-			break;
-		case 2:
-			#ifndef QT_NO_STYLE_MOTIF
-			qApp->setStyle( new QMotifStyle(true) );
-			#endif
-			break;
-		case 3:
-			#ifndef QT_NO_STYLE_MOTIFPLUS
-			qApp->setStyle( new QMotifPlusStyle(true) );
-			#endif
-			break;
-		case 4:
-			#ifndef QT_NO_STYLE_PLATINUM
-			qApp->setStyle( new QPlatinumStyle() );
-			#endif
-			break;
-		case 5:
-			#ifndef QT_NO_STYLE_SGI
-			qApp->setStyle( new QSGIStyle(true) );
-			#endif
-			break;
-		case 6:
-			#ifndef QT_NO_STYLE_WINDOWS
-			qApp->setStyle( new QWindowsStyle() );
-			#endif
-			break;
 	}
-	*/
+	else if(theme=="CDE")
+	{
+		#ifndef QT_NO_STYLE_CDE
+		qApp->setStyle( new QCDEStyle(true) );
+		#endif
+	}
+	else if(theme=="Motif")
+	{
+		#ifndef QT_NO_STYLE_MOTIF
+		qApp->setStyle( new QMotifStyle(true) );
+		#endif
+	}
+	else if(theme=="MotifPlus")
+	{
+		#ifndef QT_NO_STYLE_MOTIFPLUS
+		qApp->setStyle( new QMotifPlusStyle(true) );
+		#endif
+	}
+	else if(theme=="Platinum")
+	{
+		#ifndef QT_NO_STYLE_PLATINUM
+		qApp->setStyle( new QPlatinumStyle() );
+		#endif
+	}
+	else if(theme=="SGI")
+	{
+		#ifndef QT_NO_STYLE_SGI
+		qApp->setStyle( new QSGIStyle(true) );
+		#endif
+	}
+	else if(theme=="Windows")
+	{
+		#ifndef QT_NO_STYLE_WINDOWS
+		qApp->setStyle( new QWindowsStyle() );
+		#endif
+	}
+#else
+	QStyle * style = QStyleFactory::create(theme);
+	if (style)
+		qApp->setStyle(style);
+#endif
+	
 }
 
 void QTermFrame::hideScroll()
@@ -929,7 +1102,7 @@ void QTermFrame::defaultSetting()
 	
 	QTermConfig *pConf = new QTermConfig(addrCfg);
 
-	if( pConf->findSection("default")!=NULL )
+	if( pConf->hasSection("default") )
 		loadAddress(pConf,-1,set.param);
 
 	set.updateData(false);
@@ -952,6 +1125,7 @@ void QTermFrame::preference()
 		QTermConfig *pConf = new QTermConfig(fileCfg);
 		loadPref(pConf);
 		delete pConf;
+		setUseDock(m_pref.bTray);
 	}
 }
 
@@ -996,6 +1170,16 @@ void QTermFrame::enableMouse( )
 	specMouse->setOn( wndmgr->activeWindow()->m_bMouse );
 
 }
+
+void QTermFrame::viewImages()
+{
+	if(viewer==NULL)
+		viewer = new QTermImage(this, m_pref.strPoolPath, true);
+	else
+		viewer->updateList();
+	viewer->showNormal();
+}
+
 void QTermFrame::beep()
 {
 	wndmgr->activeWindow()->m_bBeep = !wndmgr->activeWindow()->m_bBeep;
@@ -1015,13 +1199,11 @@ void QTermFrame::reconnect()
 
 void QTermFrame::runScript()
 {
-	QMessageBox::information( this, "QTerm",
-		"Not Implemented Yet :(");
+	wndmgr->activeWindow()->runScript();
 }
 void QTermFrame::stopScript()
 {
-	QMessageBox::information( this, "QTerm",
-		"Not Implemented Yet :(");
+	wndmgr->activeWindow()->stopScript();
 }
 
 void QTermFrame::keyClicked(int id)
@@ -1040,6 +1222,7 @@ void QTermFrame::keyClicked(int id)
 		wndmgr->activeWindow()->externInput(cstrTmp.mid(1));
 	}else if(cstrTmp[0]=='1')	// script
 	{
+		wndmgr->activeWindow()->runScriptFile(cstrTmp.mid(1));
 	}
 	else if(cstrTmp[0]=='2')	// program
 	{
@@ -1047,7 +1230,32 @@ void QTermFrame::keyClicked(int id)
 	}
 }
 
-void QTermFrame::toolBarPosChanged(QToolBar* bar)
+void QTermFrame::startDown()
+{
+	if(viewer==NULL)
+		viewer = new QTermImage(this, m_pref.strPoolPath, true);
+
+	viewer->setTimeFilter(QDateTime::currentDateTime().addSecs(-1));
+}
+
+void QTermFrame::endDown()
+{
+	viewer->setTimeFilter(QDateTime());
+}
+
+void QTermFrame::previewImage(const QString& filename)
+{
+	if(viewer==NULL)
+		viewer = new QTermImage(this, m_pref.strPoolPath, true);
+	else
+		viewer->updateList();
+
+	viewer->loadImage(filename);
+
+	viewer->showNormal();
+}
+
+void QTermFrame::toolBarPosChanged(QToolBar*)
 {
 	QTermConfig conf(fileCfg);
 	
@@ -1191,123 +1399,127 @@ void QTermFrame::addMainMenu()
 	mainMenu = new QMenuBar(this);
 	
 	QPopupMenu * file = new QPopupMenu( this );
-	mainMenu->insertItem( tr("File"), file );
+	mainMenu->insertItem( tr("&File"), file );
 	file->insertItem( QPixmap(pathLib+"pic/connect.png"), 
-					tr("Connect"), this, SLOT(connectIt()) );
+					tr("&Connect"), this, SLOT(connectIt()) );
 	file->insertItem( QPixmap(pathLib+"pic/disconnect.png"),
-					tr("Disconnect"), this, SLOT(disconnect()), 0, ID_FILE_DISCONNECT );
+					tr("&Disconnect"), this, SLOT(disconnect()), 0, ID_FILE_DISCONNECT );
 
 	file->insertSeparator();
 	file->insertItem( QPixmap(pathLib+"pic/addr.png"), 
-					tr("Address Book"), this, SLOT(addressBook()), Qt::Key_F2 );
+					tr("&Address book"), this, SLOT(addressBook()), Qt::Key_F2 );
 	file->insertItem( QPixmap(pathLib+"pic/quick.png"), 
-					tr("Quick Login"),this, SLOT(quickLogin()), Qt::Key_F3 );
+					tr("&Quick login"),this, SLOT(quickLogin()), Qt::Key_F3 );
 	file->insertSeparator();
-	file->insertItem( tr("Exit"), qApp, SLOT(closeAllWindows()) );
+	file->insertItem( tr("&Exit"), this, SLOT(exitQTerm()) );
 	
 	//Edit Menu
 	QPopupMenu * edit = new QPopupMenu( this );
-	mainMenu->insertItem( tr("Edit"), edit );
+	mainMenu->insertItem( tr("&Edit"), edit );
 
 	edit->setCheckable( true );
 	edit->insertItem( QPixmap(pathLib+"pic/copy.png"),
-					tr("Copy"), this, SLOT(copy()), CTRL+Key_Insert, ID_EDIT_COPY );
+					tr("&Copy"), this, SLOT(copy()), CTRL+Key_Insert, ID_EDIT_COPY );
 	edit->insertItem( QPixmap(pathLib+"pic/paste.png"),
-					tr("Paste"), this, SLOT(paste()), SHIFT+Key_Insert, ID_EDIT_PASTE );
+					tr("&Paste"), this, SLOT(paste()), SHIFT+Key_Insert, ID_EDIT_PASTE );
 	edit->insertSeparator();
 	edit->insertItem( QPixmap(pathLib+"pic/color-copy.png"),
-					tr("Copy with Color"), this, SLOT(copyColor()), 0, ID_EDIT_COLOR );
+					tr("C&opy with color"), this, SLOT(copyColor()), 0, ID_EDIT_COLOR );
 	edit->insertItem( QPixmap(pathLib+"pic/rect.png"),
-					tr("Rectangle Select"), this, SLOT(copyRect()), 0, ID_EDIT_RECT );
-	edit->insertItem( tr("Auto Copy Select"), this, SLOT(autoCopy()),0,  ID_EDIT_AUTO );
-	edit->insertItem( tr("Paste with Word Wrap"), this, SLOT(wordWrap()), 0,  ID_EDIT_WW );
+					tr("&Rectangle select"), this, SLOT(copyRect()), 0, ID_EDIT_RECT );
+	edit->insertItem( tr("Auto copy &select"), this, SLOT(autoCopy()),0,  ID_EDIT_AUTO );
+	edit->insertItem( tr("P&aste with wordwrap"), this, SLOT(wordWrap()), 0,  ID_EDIT_WW );
 	
 	QPopupMenu * escapeMenu = new QPopupMenu(this);
 	escapeMenu->setCheckable(true);
-		escapeMenu->insertItem( tr("none"), this, SLOT(noEsc()), 0, ID_EDIT_ESC_NO );
-		escapeMenu->insertItem( tr("ESC ESC ["), this, SLOT(escEsc()), 0, ID_EDIT_ESC_ESC );
-		escapeMenu->insertItem( tr("Ctrl+U ["), this, SLOT(uEsc()), 0, ID_EDIT_ESC_U );
-		escapeMenu->insertItem( tr("Custom..."), this, SLOT(customEsc()), 0, ID_EDIT_ESC_CUS );
-	edit->insertItem( tr("Paste with color"), escapeMenu);
+		escapeMenu->insertItem( tr("&None"), this, SLOT(noEsc()), 0, ID_EDIT_ESC_NO );
+		escapeMenu->insertItem( tr("&ESC ESC ["), this, SLOT(escEsc()), 0, ID_EDIT_ESC_ESC );
+		escapeMenu->insertItem( tr("Ctrl+&U ["), this, SLOT(uEsc()), 0, ID_EDIT_ESC_U );
+		escapeMenu->insertItem( tr("&Custom..."), this, SLOT(customEsc()), 0, ID_EDIT_ESC_CUS );
+	edit->insertItem( tr("Paste &with color"), escapeMenu);
 
 	QPopupMenu * codecMenu = new QPopupMenu(this);
 	codecMenu->setCheckable(true);
-		codecMenu->insertItem( tr("GBK"), this, SLOT(gbkCodec()), 0, ID_EDIT_CODEC_GBK );
-		codecMenu->insertItem( tr("Big5"), this, SLOT(big5Codec()), 0, ID_EDIT_CODEC_BIG5 );
-	edit->insertItem( tr("Clipboard Encoding"), codecMenu);
+		codecMenu->insertItem( tr("&GBK"), this, SLOT(gbkCodec()), 0, ID_EDIT_CODEC_GBK );
+		codecMenu->insertItem( tr("&Big5"), this, SLOT(big5Codec()), 0, ID_EDIT_CODEC_BIG5 );
+	edit->insertItem( tr("Clipboard &encoding"), codecMenu);
 
 	//View menu
 	QPopupMenu * view = new QPopupMenu( this );
-	mainMenu->insertItem( tr("View"), view );
+	mainMenu->insertItem( tr("&View"), view );
 	
 	view->setCheckable( true );
 	view->insertItem( QPixmap(pathLib+"pic/fonts.png"),
-					tr("Font"), this, SLOT(font()), 0, ID_VIEW_FONT );
+					tr("&Font"), this, SLOT(font()), 0, ID_VIEW_FONT );
 	view->insertItem( QPixmap(pathLib+"pic/color.png"),
-					tr("Color"), this, SLOT(color()), 0, ID_VIEW_COLOR );
+					tr("&Color"), this, SLOT(color()), 0, ID_VIEW_COLOR );
 	view->insertItem( QPixmap(pathLib+"pic/refresh.png"),
-					tr("Refresh"), this, SLOT(refresh()), Qt::Key_F5, ID_VIEW_REFRESH );
+					tr("&Refresh"), this, SLOT(refresh()), Qt::Key_F5, ID_VIEW_REFRESH );
 	view->insertSeparator();
 	//language menu
 	langMenu = new QPopupMenu( this );
 	langMenu->setCheckable(true);
-		sEng = langMenu->insertItem( tr("English"),this,SLOT(langEnglish()));
-		sChs = langMenu->insertItem( tr("Simplified Chinese"),this,SLOT(langSimplified()));
-		sCht = langMenu->insertItem( tr("Traditional Chinese"),this,SLOT(langTraditional()));
-	view->insertItem( tr("Language"), langMenu );
-	view->insertItem( tr("UI Font"), this, SLOT(uiFont()) );
+		sEng = langMenu->insertItem( tr("&English"),this,SLOT(langEnglish()));
+		sChs = langMenu->insertItem( tr("&Simplified Chinese"),this,SLOT(langSimplified()));
+		sCht = langMenu->insertItem( tr("&Traditional Chinese"),this,SLOT(langTraditional()));
+	view->insertItem( tr("&Language"), langMenu );
+	view->insertItem( tr("&UI font"), this, SLOT(uiFont()) );
 
 	themesMenu = new QPopupMenu( this );
 	themesMenu->setCheckable( TRUE );
 	connect( themesMenu, SIGNAL( aboutToShow() ),
 	     this, SLOT( themesMenuAboutToShow() ) );
-	view->insertItem( tr("Thems"), themesMenu );
+	view->insertItem( tr("&Themes"), themesMenu );
 
-	view->insertItem( tr("Fullscren"), this, SLOT(fullscreen()), Qt::Key_F6, ID_VIEW_FULL );
-	view->insertItem( tr("Boss Color"), this, SLOT(bosscolor()), Qt::Key_F12, ID_VIEW_BOSS );
+	QAccel *a = new QAccel(this);
+	a->connectItem(a->insertItem(Qt::Key_F6), this, SLOT(fullscreen()) );
+	view->insertItem( tr("&Fullscreen"), this, SLOT(fullscreen()), Qt::Key_F6, ID_VIEW_FULL );
+
+	view->insertItem( tr("Boss &Color"), this, SLOT(bosscolor()), Qt::Key_F12, ID_VIEW_BOSS );
 
 	view->insertSeparator();
 	QPopupMenu *scrollMenu = new QPopupMenu(this);
-		scrollMenu->insertItem( tr("Hide"), this, SLOT(hideScroll()), 0, ID_VIEW_SCROLL_HIDE );
-		scrollMenu->insertItem( tr("Left"), this, SLOT(leftScroll()), 0, ID_VIEW_SCROLL_LEFT );
-		scrollMenu->insertItem( tr("Right"), this, SLOT(rightScroll()), 0, ID_VIEW_SCROLL_RIGHT );
-	view->insertItem( tr("ScrollBar"), scrollMenu );
-	view->insertItem( tr("Status Bar"), this, SLOT(showStatusBar()), 0, ID_VIEW_STATUS );
-	view->insertItem( tr("Switch Bar"), this, SLOT(showSwitchBar()), 0, ID_VIEW_SWITCH );
+		scrollMenu->insertItem( tr("&Hide"), this, SLOT(hideScroll()), 0, ID_VIEW_SCROLL_HIDE );
+		scrollMenu->insertItem( tr("&Left"), this, SLOT(leftScroll()), 0, ID_VIEW_SCROLL_LEFT );
+		scrollMenu->insertItem( tr("&Right"), this, SLOT(rightScroll()), 0, ID_VIEW_SCROLL_RIGHT );
+	view->insertItem( tr("&ScrollBar"), scrollMenu );
+	view->insertItem( tr("Status &Bar"), this, SLOT(showStatusBar()), 0, ID_VIEW_STATUS );
+	view->insertItem( tr("S&witch Bar"), this, SLOT(showSwitchBar()), 0, ID_VIEW_SWITCH );
 
 	
 	// Option Menu	
 	QPopupMenu * option = new QPopupMenu( this );
-	mainMenu->insertItem( tr("Option"), option );
+	mainMenu->insertItem( tr("&Option"), option );
 
-	option->insertItem( tr("Setting for Currrent Session"), this, SLOT(setting()), 0, ID_OPTION_CURRENT );
+	option->insertItem( tr("&Setting for currrent session"), this, SLOT(setting()), 0, ID_OPTION_CURRENT );
 	option->insertSeparator();
-	option->insertItem( tr("Default Setting"), this, SLOT(defaultSetting()) );
-	option->insertItem( tr("Preference"), this, SLOT(preference()) );
+	option->insertItem( tr("&Default setting"), this, SLOT(defaultSetting()) );
+	option->insertItem( tr("&Preference"), this, SLOT(preference()) );
 	
 	// Special
 	QPopupMenu * spec = new QPopupMenu( this );
-	mainMenu->insertItem( tr("Special"), spec );
+	mainMenu->insertItem( tr("&Special"), spec );
 	spec->insertItem( QPixmap(pathLib+"pic/article.png"),
-					tr("Copy Article"), this, SLOT(copyArticle()), Qt::Key_F9, ID_SPEC_ARTICLE );
+					tr("&Copy article"), this, SLOT(copyArticle()), Qt::Key_F9, ID_SPEC_ARTICLE );
 	spec->setCheckable( true );
 	spec->insertItem( QPixmap(pathLib+"pic/anti-idle.png"),
-					tr("Anti Idle"), this, SLOT(antiIdle()), 0, ID_SPEC_ANTI );
+					tr("Anti &idle"), this, SLOT(antiIdle()), 0, ID_SPEC_ANTI );
 	spec->insertItem(QPixmap(pathLib+"pic/auto-reply.png"), 
-					tr("Auto Reply"), this, SLOT(autoReply()), 0, ID_SPEC_AUTO );
+					tr("Auto &reply"), this, SLOT(autoReply()), 0, ID_SPEC_AUTO );
 	spec->insertItem( QPixmap(pathLib+"pic/message.png"),
-					tr("View Messages"), this, SLOT(viewMessages()), Qt::Key_F10, ID_SPEC_MESSAGE );
+					tr("&View messages"), this, SLOT(viewMessages()), Qt::Key_F10, ID_SPEC_MESSAGE );
 	spec->insertItem( QPixmap(pathLib+"pic/sound.png"),
-					tr("Beep "), this, SLOT(beep()), 0, ID_SPEC_BEEP );
+					tr("&Beep "), this, SLOT(beep()), 0, ID_SPEC_BEEP );
 	spec->insertItem( QPixmap(pathLib+"pic/mouse.png"),
-					tr("Mouse Support"), this, SLOT(enableMouse()), 0, ID_SPEC_MOUSE );
+					tr("&Mouse support"), this, SLOT(enableMouse()), 0, ID_SPEC_MOUSE );
+	spec->insertItem(tr("&Image viewer"), this, SLOT(viewImages()));
 
 	
 	//Script
 	QPopupMenu * script = new QPopupMenu( this );
-	mainMenu->insertItem( tr("Script"), script );
-	script->insertItem( tr("Run..."), this, SLOT(runScript()), Qt::Key_F7, ID_SCRIPT_RUN );
-	script->insertItem( tr("Stop"), this, SLOT(stopScript()), Qt::Key_F8, ID_SCRIPT_STOP );
+	mainMenu->insertItem( tr("Scrip&t"), script );
+	script->insertItem( tr("&Run..."), this, SLOT(runScript()), Qt::Key_F7, ID_SCRIPT_RUN );
+	script->insertItem( tr("&Stop"), this, SLOT(stopScript()), Qt::Key_F8, ID_SCRIPT_STOP );
 
 	//Window menu
 	windowsMenu = new QPopupMenu( this );
@@ -1315,14 +1527,14 @@ void QTermFrame::addMainMenu()
 	connect( windowsMenu, SIGNAL( aboutToShow() ),
 	     this, SLOT( windowsMenuAboutToShow() ) );
 
-	mainMenu->insertItem( tr("Windows"), windowsMenu );
+	mainMenu->insertItem( tr("&Windows"), windowsMenu );
 	mainMenu->insertSeparator();
 
 	//Help menu
 	QPopupMenu * help = new QPopupMenu( this );
-	mainMenu->insertItem( tr("Help"), help );
-	help->insertItem( tr("About Qterm"), this, SLOT(aboutQTerm()), Qt::Key_F1 );
-	help->insertItem( tr("QTerm's Homepage"),this,SLOT(homepage()));
+	mainMenu->insertItem( tr("&Help"), help );
+	help->insertItem( tr("About &QTerm"), this, SLOT(aboutQTerm()), Qt::Key_F1 );
+	help->insertItem( tr("QTerm's &Homepage"),this,SLOT(homepage()));
 
 }
 
@@ -1434,21 +1646,216 @@ QCString QTermFrame::valueToString(bool shown, int dock, int index, bool nl, int
 	return cstr;
 }
 
-void QTermFrame::popupFocusIn(QTermWindow *w)
+void QTermFrame::popupFocusIn(QTermWindow *)
 {
-	setFocus();
-	w->setFocus();
+	// bring to font
+	if(isHidden()) {
+		show();
+	}
+	if(isMinimized()) {
+		if(isMaximized())
+			showMaximized();
+		else
+			showNormal();
+	}
+	raise();
+	setActiveWindow();
 }
 
-void QTermFrame::insertThemeItem(int i)
+void QTermFrame::insertThemeItem(QString themeitem)
 {
-	char *style[]={"Default", "CDE", "Motif", 
-			"Motif Plus", "Platinum", "SGI", "Windows"};
+	//const char *style[]={"Default", "CDE", "Motif", 
+			//"Motif Plus", "Platinum", "SGI", "Windows"};
 	int id;
 
-	id = themesMenu->insertItem(style[i], this, SLOT(themesMenuActivated(int)));
-	themesMenu->setItemParameter( id, i );
-	themesMenu->setItemChecked( id, i==theme );
+	id = themesMenu->insertItem(themeitem, this, SLOT(themesMenuActivated(int)));
+	themesMenu->setItemParameter( id, id );
+	themesMenu->setItemChecked( id, themeitem==theme );
+}
+
+void QTermFrame::setUseDock(bool use)
+{
+	if(use == false) {
+		if(tray) {
+			delete tray;
+			tray = 0;
+			delete trayMenu;
+			trayMenu = 0;
+		}
+		return;
+	}
+
+	if(tray)
+		return;
+
+	trayMenu = new QPopupMenu;
+	connect(trayMenu, SIGNAL(aboutToShow()), SLOT(buildTrayMenu()));
+
+	
+	tray = new MTray( pathLib+"pic/qterm_tray.png", "QTerm", trayMenu, this);
+	connect(tray, SIGNAL(clicked(const QPoint &, int)), SLOT(trayClicked(const QPoint &, int)));
+	connect(tray, SIGNAL(doubleClicked(const QPoint &)), SLOT(trayDoubleClicked()));
+	connect(tray, SIGNAL(closed()), this, SLOT(exitQTerm()));
+
+	tray->show();
+}
+
+void QTermFrame::buildTrayMenu()
+{
+	if(!trayMenu)
+		return;
+	trayMenu->clear();
+
+	if(isHidden())
+		trayMenu->insertItem(tr("Show"), this, SLOT(trayShow()));
+	else
+		trayMenu->insertItem(tr("Hide"), this, SLOT(trayHide()));
+	trayMenu->insertSeparator();	
+	trayMenu->insertItem(tr("About"), this, SLOT(aboutQTerm()));
+	trayMenu->insertItem(tr("Exit"), this, SLOT(exitQTerm()));
+}
+
+void QTermFrame::trayClicked(const QPoint &, int)
+{
+	if(isHidden())
+		trayShow();
+	else
+		trayHide();
+}
+
+void QTermFrame::trayDoubleClicked()
+{
+	if(isHidden())
+		trayShow();
+	else
+		trayHide();
+}
+
+void QTermFrame::trayShow()
+{
+	show();
+	// bring to font
+	if(isHidden()) {
+		show();
+	}
+	if(isMinimized()) {
+		if(isMaximized())
+			showMaximized();
+		else
+			#ifdef Q_OS_MACX
+			showMaximized();
+			#else
+			showNormal();
+			#endif
+	}
+	raise();
+	setActiveWindow();
+}
+
+void QTermFrame::trayHide()
+{
+	hide();
 }
 
 
+//----------------------------------------------------------------------------
+// MTray
+//----------------------------------------------------------------------------
+MTray::MTray(const QString &icon, const QString &tip, QPopupMenu *popup, QObject *parent)
+:QObject(parent)
+{
+	ti = new TrayIcon(QPixmap(icon), tip, popup);
+	connect(ti, SIGNAL(clicked(const QPoint &, int)), SIGNAL(clicked(const QPoint &, int)));
+	connect(ti, SIGNAL(doubleClicked(const QPoint &)), SIGNAL(doubleClicked(const QPoint &)));
+	connect(ti, SIGNAL(closed()), SIGNAL(closed()));
+	ti->show();
+}
+
+MTray::~MTray()
+{
+	delete ti;
+}
+
+void MTray::setToolTip(const QString &str)
+{
+	ti->setToolTip(str);
+}
+
+void MTray::setImage(const QImage &i)
+{
+	ti->setIcon(makeIcon(i));
+}
+
+void MTray::show()
+{
+	ti->show();
+}
+
+void MTray::hide()
+{
+	ti->hide();
+}
+
+QPixmap MTray::makeIcon(const QImage &_in)
+{
+#ifdef Q_WS_X11
+		// on X11, the KDE dock is 22x22.  let's make our icon "seem" bigger.
+		QImage real(22,22,32);
+		//QImage in = _in.convertToImage();
+		QImage in = _in;
+		in.detach();
+		real.fill(0);
+		real.setAlphaBuffer(true);
+
+		// make sure it is no bigger than 16x16
+		if(in.width() > 16 || in.height() > 16)
+			in = in.smoothScale(16,16);
+
+		int xo = (real.width() - in.width()) / 2;
+		int yo = (real.height() - in.height()) / 2;
+
+		int n, n2;
+
+		// draw a dropshadow
+		for(n2 = 0; n2 < in.height(); ++n2) {
+			for(n = 0; n < in.width(); ++n) {
+				if(qAlpha(in.pixel(n,n2))) {
+					int x = n + xo + 2;
+					int y = n2 + yo + 2;
+					real.setPixel(x, y, qRgba(0x80,0x80,0x80,0xff));
+				}
+			}
+		}
+		// draw the image
+		for(n2 = 0; n2 < in.height(); ++n2) {
+			for(n = 0; n < in.width(); ++n) {
+				if(qAlpha(in.pixel(n,n2))) {
+					QRgb c = in.pixel(n, n2);
+					real.setPixel(n + xo, n2 + yo, qRgba(qRed(c), qGreen(c), qBlue(c), 0xff));
+				}
+			}
+		}
+		// create the alpha layer
+		for(n2 = real.height()-2; n2 >= 0; --n2) {
+			for(n = 0; n < real.width(); ++n) {
+				uint c = real.pixel(n, n2);
+				if(c > 0) {
+					QRgb old = real.pixel(n, n2);
+					real.setPixel(n, n2, qRgba(qRed(old), qGreen(old), qBlue(old), 0xff));
+				}
+			}
+		}
+
+		QPixmap icon;
+		icon.convertFromImage(real);
+		return icon;
+	//}
+#else
+	QPixmap icon;
+	icon.convertFromImage(_in);
+	return icon;
+#endif
+}
+
+
+#include <qtermframe.moc>
