@@ -3,6 +3,7 @@
 
 
 #include <qmainwindow.h>
+#include "statusBar.h"
 
 class QLineEdit;
 class QToolBar;
@@ -16,6 +17,8 @@ class QVBox;
 class QFontDialog;
 class QTermParam;
 class QTermConfig;
+class TrayIcon;
+class QTermImage;
 
 struct QTermPref
 {
@@ -29,11 +32,45 @@ struct QTermPref
 	QString strHttp;
 	int		nBeep;
 	QString strWave;
+	int		nMethod;
+	QString strPlayer;
 	bool	bUrl;
 	bool	bAutoCopy;
 	bool	bAA;
+	bool	bTray;
+	bool	bClearPool;
+	QString strZmPath;
+	QString strPoolPath;
+	QString strImageViewer;
 };
 
+class MTray : public QObject
+{
+	Q_OBJECT
+public:
+	MTray(const QString &icon, const QString &tip, QPopupMenu *popup, QObject *parent=0);
+	~MTray();
+
+	void setToolTip(const QString &);
+	void setImage(const QImage &);
+
+signals:
+	void clicked(const QPoint &, int);
+	void doubleClicked(const QPoint &);
+	void closed();
+
+public slots:
+	void show();
+	void hide();
+
+private:
+	TrayIcon *ti;
+	QTimer *trayTimer;
+
+	QPixmap makeIcon(const QImage &_in);
+};
+
+
 class QTermFrame : public QMainWindow
 {
 	Q_OBJECT
@@ -50,14 +87,17 @@ signals:
 	void updateScroll();
 	void updateStatusBar(bool);
 
+public slots:
+	void previewImage(const QString&);
+
 protected slots:
-	void dialogInput();
 	void keyClicked(int);
 	void toolBarPosChanged(QToolBar*);
 	// Menu
 	void addressBook();
 	void quickLogin();
-	
+	void exitQTerm();
+
 	void selectionChanged(int);
 	void aboutQTerm();
 	void homepage();
@@ -104,6 +144,8 @@ protected slots:
     void beep();
 	void reconnect();
 	void keySetup();
+
+	void viewImages();
 	
 	void themesMenuAboutToShow();
 	void themesMenuActivated(int);
@@ -112,9 +154,18 @@ protected slots:
 	void connectMenuActivated(int);
 	void popupConnectMenu();
 	void connectMenuAboutToHide();
+	
+	void trayClicked(const QPoint &, int);
+	void trayDoubleClicked();	
+	void trayHide();
+	void trayShow();
+	void buildTrayMenu();
 
 	void switchWin(int);
 	void paintEvent( QPaintEvent * );
+
+	void startDown();
+	void endDown();
 public:
 	QTabBar *tabBar;
 	QTermWndMgr * wndmgr;
@@ -129,23 +180,24 @@ public:
 	bool m_bStatusBar;
 	
 	int m_nScrollPos;	// 0--hide 1--LEFT 2--RIGHT
+
+	QWorkspace* ws;
 protected:   
 	//variables
-	QWorkspace* ws;
 	QTermTimeLabel *labelTime;
 	QPopupMenu *windowsMenu;
 	QPopupMenu *themesMenu;
 	int sEng,sChs,sCht;
-	int theme;
+	QString theme;
 
-//	QDockWindow * dock;
-	QLineEdit * input;
 	QToolBar * key;
 
 	QPopupMenu * escapeMenu;
 	QPopupMenu * langMenu;	
 	QPopupMenu * connectMenu;
 
+	QTerm::StatusBar * m_pStatusBar;
+
 	QToolButton *connectButton, *disconnectButton,
 				*editRect, *editColor,
 				*specAnti, *specAuto, *specMouse, *specBeep, *specReconnect;
@@ -155,8 +207,14 @@ protected:
 	
 	bool m_bFullScreen;
 	bool m_bSwitchBar;
+
+	MTray *tray;
+	QPopupMenu *trayMenu;
+
+	QTermImage *viewer;
 	//function
-	QTermWindow * newWindow( const QTermParam& param, int index=-1 );
+	//QTermWindow * newWindow( const QTermParam& param, int index=-1 );
+	void newWindow( const QTermParam& param, int index=-1 );
 	void closeEvent(QCloseEvent * );
 	void selectStyleMenu(int ,int );
 	void iniSetting();
@@ -167,9 +225,12 @@ protected:
 	void addMainTool();
 
 	void updateKeyToolBar();
+	
+	bool eventFilter(QObject*, QEvent *);
 
 	QCString valueToString(bool, int, int, bool, int);
-	void insertThemeItem(int);
+	void insertThemeItem(QString);
+	void setUseDock(bool);
 };
 
 #endif	//QTERMFRAME_H
