@@ -10,28 +10,32 @@ AUTHOR:        kingson fiasco
  *******************************************************************************/
 #include "qterm.h"
 
+#include "qtermwindow.h"
 #include "qtermscreen.h"
 #include "qtermtextline.h"
 #include "qtermconvert.h"
 #include "qtermbuffer.h"
-#include "qtermwindow.h"
 #include "qtermbbs.h"
 #include "qtermframe.h"
 #include "qtermwndmgr.h"
 #include "qtermparam.h"
 #include "qtermtelnet.h"
+#include "qtermconfig.h"
 
 #include <qaccel.h>
 #include <qapplication.h>
 #include <qpainter.h>
 #include <qpixmap.h>
+#include <qimage.h>
 #include <qtextcodec.h>
 #include <qfontmetrics.h>
 #include <qtimer.h>
 #include <qevent.h>
 #include <qpoint.h>
 #include <qclipboard.h>
+#include <qfile.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 /* ------------------------------------------------------------------------ */
 /*                                                                          */
@@ -48,7 +52,11 @@ QTermScreen::QTermScreen( QWidget *parent, QTermBuffer *buffer, QTermParam *para
 
 	setFocusPolicy(ClickFocus);	
 
-	initColorTable();
+	#if (QT_VERSION>=0x030200)
+	setInputMethodEnabled(true);
+	#endif
+
+	setSchema();
 
 	initFontMetrics();
 
@@ -66,11 +74,14 @@ QTermScreen::QTermScreen( QWidget *parent, QTermBuffer *buffer, QTermParam *para
 	m_cursorTimer = new QTimer(this);
 	connect(m_cursorTimer, SIGNAL(timeout()), this, SLOT(cursorEvent()));
 
+	m_inputContent = NULL;
+
 	// the scrollbar
 	m_scrollBar = new QScrollBar(this);
 	m_scrollBar->setCursor( arrowCursor );
 	m_nStart = 0;
 	m_nEnd = m_pBuffer->line()-1;
+	m_pBlinkLine = new bool[m_nEnd - m_nStart + 1];
 	m_scrollBar->setRange( 0, 0 );
 	m_scrollBar->setLineStep(1);
 	m_scrollBar->setPageStep( m_pBuffer->line() );
@@ -99,27 +110,40 @@ QTermScreen::QTermScreen( QWidget *parent, QTermBuffer *buffer, QTermParam *para
 		m_pCodec = QTextCodec::codecForName("GBK");
 	else
 		m_pCodec = QTextCodec::codecForName("Big5");
-	
-	m_nPxmType = 0;
 }
 
 QTermScreen::~QTermScreen()
 {
+	delete [] m_pBlinkLine;
 	delete m_blinkTimer;
 	delete m_cursorTimer;
+	delete m_inputContent;
 }
 
-
 // focus event received
 void QTermScreen::focusInEvent( QFocusEvent * )
 {
+
+//	#if (QT_VERSION>0x030300) // windows get minimized when closing
+//	or switching by ctrl+tab. blame me for this...
+	if(m_pWindow->isMaximized()&&m_pWindow->m_pFrame->wndmgr->afterRemove())
+	{
+		m_pWindow->showNormal();
+		m_pWindow->showMaximized();
+	}
+//	#endif
+
 	m_pWindow->m_pFrame->wndmgr->activateTheTab( m_pWindow );
 	
 #if (QT_VERSION<300)
-	m_pAccel->connectItem(idPrevPage=m_pAccel->insertItem(Key_PageUp+SHIFT), this, SLOT(prevPage()));
-	m_pAccel->connectItem(idNextPage=m_pAccel->insertItem(Key_PageDown+SHIFT), this, SLOT(nextPage()));
-	m_pAccel->connectItem(idPrevLine=m_pAccel->insertItem(Key_Up+SHIFT), this, SLOT(prevLine()));
-	m_pAccel->connectItem(idNextLine=m_pAccel->insertItem(Key_Down+SHIFT), this, SLOT(nextLine()));
+	m_pAccel->connectItem(idPrevPage=m_pAccel->insertItem(Key_PageUp+SHIFT),
+					this, SLOT(prevPage()));
+	m_pAccel->connectItem(idNextPage=m_pAccel->insertItem(Key_PageDown+SHIFT),
+					this, SLOT(nextPage()));
+	m_pAccel->connectItem(idPrevLine=m_pAccel->insertItem(Key_Up+SHIFT), 
+					this, SLOT(prevLine()));
+	m_pAccel->connectItem(idNextLine=m_pAccel->insertItem(Key_Down+SHIFT),
+					this, SLOT(nextLine()));
 #endif
 }
 
@@ -176,7 +200,7 @@ void QTermScreen::blinkEvent()
 	if(m_hasBlink)
 	{
 		m_blinkScreen = !m_blinkScreen;
-		repaint(false);
+		blinkScreen();
 	}
 }
 
@@ -191,20 +215,16 @@ void QTermScreen::resizeEvent( QResizeEvent *  )
 	
 	if( m_pParam->m_bAutoFont )
 	{
-		updateFont();
+		updateFont2();
 	}
-	else
+	else 
 	{
-		/*
-		// FIXME this still not work
 		int cx = m_rcClient.width()/m_nCharWidth;
 		int cy = m_rcClient.height()/m_nCharHeight;
-		if( cx>0 && cy>0 )
+		if( cx>10 && cy>10 )
 		{
 			m_pBuffer->setSize( cx, cy );
-			m_pWindow->m_pTelnet->setWindowSize( cx, cy );
 		}
-		*/
 	}
 }
 
@@ -232,9 +252,11 @@ void QTermScreen::mousePressEvent( QMouseEvent * me )
 }
 void QTermScreen::mouseMoveEvent( QMouseEvent * me)
 {
-
+#ifdef Q_OS_MACX
+	m_pWindow->mouseMoveEvent(me);
+#else
 	QApplication::sendEvent(m_pWindow, me);
-
+#endif
 }
 
 void QTermScreen::mouseReleaseEvent( QMouseEvent * me )
@@ -261,19 +283,19 @@ void QTermScreen::initFontMetrics()
 {
 	if(m_pParam->m_bAutoFont)
 		m_pFont = new QFont(m_pParam->m_strFontName); 
-	else
+	else {
 		m_pFont = new QFont(m_pParam->m_strFontName, QMAX(8,m_pParam->m_nFontSize) ); 
+		QFontMetrics *fm = new QFontMetrics( *m_pFont );
+
+		getFontMetrics(fm);
+		delete fm;
+	}
 
 	#if (QT_VERSION >= 300 )
 	m_pFont->setStyleHint(QFont::System, 
-				m_pWindow->m_pFrame->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+		m_pWindow->m_pFrame->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
 	#endif
 	
-	QFontMetrics *fm = new QFontMetrics( *m_pFont );
-	m_nCharWidth  = fm->width('W');
-	m_nCharHeight = fm->height();
-	m_nCharAscent = fm->ascent();
-	delete fm;
 }
 
 void QTermScreen::setDispFont( const QFont& font)
@@ -281,14 +303,21 @@ void QTermScreen::setDispFont( const QFont& font)
 	delete m_pFont;
 	if(m_pParam->m_bAutoFont)
 		m_pFont = new QFont(font.family());
-	else
+	else {
 		m_pFont = new QFont(font);
+		
+		QFontInfo fi(*m_pFont);
+		int nSize = fi.pixelSize();
+		QFontMetrics *fm = new QFontMetrics( *m_pFont );
+		getFontMetrics(fm);
+		delete fm;
 
-	QFontMetrics *fm = new QFontMetrics( *m_pFont );
-	m_nCharWidth  = fm->width('W');
-	m_nCharHeight = fm->height();
-	m_nCharAscent = fm->ascent();
-	delete fm;
+	}
+
+	#if (QT_VERSION >= 300 )
+	m_pFont->setStyleHint(QFont::System, 
+		m_pWindow->m_pFrame->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+	#endif
 
 }
 
@@ -297,6 +326,7 @@ QFont QTermScreen::getDispFont( )
 	return *m_pFont;
 }
 
+
 void QTermScreen::updateFont()
 {
 	QString strFamily = m_pFont->family();
@@ -313,7 +343,15 @@ void QTermScreen::updateFont()
 		m_pFont->setPixelSize( nPixelSize );
 		
 		QFontMetrics fm( *m_pFont );
-		m_nCharWidth = fm.width( 'W');
+/* 
+   Use the width of char 'W' or the width of Chinese charactor to calculate
+   the display. use nPixelSize as a reference size. 0x4e00 stand for Chinese
+   one ;)
+*/
+		if (abs(nPixelSize - fm.width(QChar(0x4e00))) < abs(nPixelSize - fm.width('W') * 2))
+			m_nCharWidth  = (fm.width(QChar(0x4e00)) + 1)/2;
+		else
+			m_nCharWidth  = fm.width('W');
 		m_nCharHeight = fm.height();
 		m_nCharAscent = fm.ascent();
 		m_nCharDescent = fm.descent();
@@ -334,21 +372,82 @@ void QTermScreen::updateFont()
 									m_pWindow->m_pFrame->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
 	#endif
 	QFontMetrics fm( *m_pFont );
-	m_nCharWidth = fm.width( 'W');
+	if (abs(nPixelSize - 1 - fm.width(QChar(0x4e00))) < abs(nPixelSize - 1 - fm.width('W') * 2))
+		m_nCharWidth  = (fm.width(QChar(0x4e00)) + 1)/2;
+	else
+		m_nCharWidth  = fm.width('W');
+/*
+   Don't trust the font width if it's nonsense, we just guess it from the 
+   pixelsize, mostly because some system(redhat and some debian) return the 
+   wrong width of Chinese charactors, for using of variable fonts, we can
+   only use nPixelsize
+*/
+	if (abs(nPixelSize - m_nCharWidth * 2) > (nPixelSize > 16 ? 3 : 2))
+		m_nCharWidth = (nPixelSize + 1)/2;
 	m_nCharHeight = fm.height();
 	m_nCharAscent = fm.ascent();
 	m_nCharDescent = fm.descent();
 
 }
 
+void QTermScreen::updateFont2()
+{
+	int nPixelSize;
+	int nIniSize = QMAX(8,QMIN(m_rcClient.height()/m_pBuffer->line(),
+					m_rcClient.width()*2/m_pBuffer->columns()));
+
+	for( nPixelSize=nIniSize-3; nPixelSize<=nIniSize+3; nPixelSize++)
+	{
+		m_pFont->setPixelSize( nPixelSize );
+		
+		QFontMetrics fm( *m_pFont );
+		getFontMetrics(&fm);
+		if( (m_pBuffer->line()*m_nCharHeight)>m_rcClient.height()
+			|| (m_pBuffer->columns()*m_nCharWidth)>m_rcClient.width()  )
+		{
+			while(nPixelSize>5)
+			{
+				nPixelSize--;
+				m_pFont->setPixelSize( nPixelSize );
+		
+				QFontMetrics fm2( *m_pFont );
+				getFontMetrics(&fm2);
+				if( (m_pBuffer->line()*m_nCharHeight)<m_rcClient.height()
+				&& (m_pBuffer->columns()*m_nCharWidth)<m_rcClient.width()  )
+					break;
+			}
+			break;	
+		}
+	}
+	#if (QT_VERSION >= 300 )
+	m_pFont->setStyleHint(QFont::System, 
+		m_pWindow->m_pFrame->m_pref.bAA ? QFont::PreferAntialias : QFont::NoAntialias);
+	#endif
+
+}
+
+void QTermScreen::getFontMetrics(QFontMetrics *fm)
+{
+	float cn=fm->width(QChar(0x4e00));
+	float en=fm->width('W');
+	if(en/cn<0.7) // almost half
+		m_nCharWidth = QMAX((cn+1)/2,en);
+	else
+		m_nCharWidth = (QMAX(en,cn)+1)/2;
+
+	m_nCharHeight = fm->height();
+	m_nCharAscent = fm->ascent();
+	m_nCharDescent = fm->descent();
+}
 
 /* ------------------------------------------------------------------------ */
 /*	                                                                        */
 /* 	                               Colors                                   */
 /*	                                                                        */
 /* ------------------------------------------------------------------------ */
-void QTermScreen::initColorTable()
+void QTermScreen::setSchema()
 {
+// the default color table
 	m_color[0]  = Qt::black;
 	m_color[1]  = Qt::darkRed;
 	m_color[2]  = Qt::darkGreen;
@@ -366,39 +465,61 @@ void QTermScreen::initColorTable()
 	m_color[14] = Qt::cyan;
 	m_color[15] = Qt::white;
 
+	m_nPxmType = 0;
+
+
+
+// if we have schema defined
+	if(QFile::exists(m_pParam->m_strSchemaFile))
+	{
+
+//		printf("schema %s loaded sucessfully\n", m_pParam->m_strSchemaFile);
+		QTermConfig *pConf = new QTermConfig(m_pParam->m_strSchemaFile);
+
+		m_color[0].setNamedColor(pConf->getItemValue("color","color0"));
+		m_color[1].setNamedColor(pConf->getItemValue("color","color1"));
+		m_color[2].setNamedColor(pConf->getItemValue("color","color2"));
+		m_color[3].setNamedColor(pConf->getItemValue("color","color3"));
+		m_color[4].setNamedColor(pConf->getItemValue("color","color4"));
+		m_color[5].setNamedColor(pConf->getItemValue("color","color5"));
+		m_color[6].setNamedColor(pConf->getItemValue("color","color6"));
+		m_color[7].setNamedColor(pConf->getItemValue("color","color7"));
+		m_color[8].setNamedColor(pConf->getItemValue("color","color8"));
+		m_color[9].setNamedColor(pConf->getItemValue("color","color9"));
+		m_color[10].setNamedColor(pConf->getItemValue("color","color10"));
+		m_color[11].setNamedColor(pConf->getItemValue("color","color11"));
+		m_color[12].setNamedColor(pConf->getItemValue("color","color12"));
+		m_color[13].setNamedColor(pConf->getItemValue("color","color13"));
+		m_color[14].setNamedColor(pConf->getItemValue("color","color14"));
+		m_color[15].setNamedColor(pConf->getItemValue("color","color15"));
+
+		// bg type
+		QString strTmp = pConf->getItemValue("image", "type");
+		m_nPxmType = strTmp.toInt();
+		
+		// fade effect
+		QColor fadecolor;
+		fadecolor.setNamedColor(pConf->getItemValue("image","fade"));
+		strTmp = pConf->getItemValue("image", "alpha");
+		float alpha = strTmp.toFloat();
+
+		// get the image name
+		if(QFile::exists(pConf->getItemValue("image","name")) && m_nPxmType>1) // valid image name and type
+		{
+			m_pxmBg = QPixmap(pConf->getItemValue("image","name"));
+			QImage ima(m_pxmBg.convertToImage());
+			ima = fade(ima, alpha, fadecolor);
+			m_pxmBg.convertFromImage(ima);
+
+		}
+		delete pConf;
+	}
+
+// override schema using user defined Fg/Bg color	
 	m_color[0]  = m_pParam->m_clrBg;
 	m_color[7]  = m_pParam->m_clrFg;
 }
-void QTermScreen::setColorTable( const QColor table[] )
-{
-	 for (int i = 0; i < 16; i++) 
-		 m_color[i] = table[i];
-}
 
-const QColor * QTermScreen::getColorTable()
-{
-	return m_color;
-}
-
-void QTermScreen::setFgColor( const QColor& colorFg )
-{
-	m_color[7]=colorFg;
-}
-
-void QTermScreen::setBgColor( const QColor& colorBg )
-{
-	m_color[0]=colorBg;
-}
-
-QColor QTermScreen::getFgColor( )
-{
-	return m_color[7];
-}
-
-QColor QTermScreen::getBgColor( )
-{
-	return m_color[0];
-}
 
 /* ------------------------------------------------------------------------ */
 /*	                                                                        */
@@ -519,6 +640,11 @@ void QTermScreen::bufferSizeChanged()
 
 	connect(m_scrollBar, SIGNAL(valueChanged(int)), 
 			this, SLOT(scrollChanged(int)));
+
+	delete []m_pBlinkLine;
+	m_pBlinkLine = new bool[m_nEnd - m_nStart + 1];
+
+	scrollLine(0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -589,12 +715,33 @@ void QTermScreen::setBgPxm( const QPixmap& pixmap, int nType)
 	}
 }
 
+void QTermScreen::blinkScreen()
+{
+	QPainter painter;
+	painter.begin(this);
+	int startx;
+	for (int index=m_nStart; index<=m_nEnd; index++)
+		if (m_pBlinkLine[index - m_nStart]) {
+			QTermTextLine *pTextLine = m_pBuffer->at(index);
+			//erase(mapToRect(0, index, pTextLine->getLength(), 1));
+		        //drawLine(painter, index);
+			uint linelength = pTextLine->getLength();
+			QCString attr = pTextLine->getAttr();
+			for (uint i = 0; i < linelength; ++i)
+				if (GETBLINK(attr.at(i))) {
+					startx = i;
+					while(GETBLINK(attr.at(i))&& i < linelength)
+						++i;
+					drawLine(painter, index, startx, i, false);
+				}
+		}
+}
+
 // refresh the screen when 
 //	1. received new contents form server
 //	2. scrolled by user
 void QTermScreen::refreshScreen( )
 {
-
 	if( m_cursorTimer->isActive() ) m_cursorTimer->stop();
 
 	if( m_blinkTimer->isActive() ) m_blinkTimer->stop();
@@ -607,6 +754,8 @@ void QTermScreen::refreshScreen( )
 
 	QPainter painter;
 	painter.begin(this);
+	
+	
 	for(int index=m_nStart; index<=m_nEnd; index++ )
 	{
 		if ( index>=m_pBuffer->lines() )
@@ -617,17 +766,25 @@ void QTermScreen::refreshScreen( )
 		}
 		QTermTextLine *pTextLine = m_pBuffer->at(index);
 		
-		if(pTextLine->hasBlink())
+		if(pTextLine->hasBlink()){
 			m_hasBlink = true;
+			m_pBlinkLine[index - m_nStart] = true;
+		}
+		else
+			m_pBlinkLine[index - m_nStart] = false;
 		
 		if(!pTextLine->isChanged(startx, endx))
 			continue;
-		
-		QRect rect = mapToRect(startx, index, endx, 1);
-		
-		erase(rect);
+			
+/* 
+  Finally get around this for pku & ytht, don't know why some weird things
+  happened when only erase and draw the changed part. 
+*/
+		startx = testChar(startx, index);
 
-		drawLine( painter, index );
+		erase(mapToRect(startx, index, -1, 1));
+
+		drawLine( painter, index, startx);
 
 		pTextLine->clearChange();
 	}
@@ -646,6 +803,56 @@ void QTermScreen::refreshScreen( )
 	if(m_hasBlink)	m_blinkTimer->start(1000);
 
 }
+
+int QTermScreen::testChar(int startx, int index)
+{
+	QTermTextLine *pTextLine = m_pBuffer->at(index);
+	QString strDisplay;
+	QString strTest;
+	QCString cstrDisplay;
+	QCString cstrTest;
+	
+	if ( index >= m_pBuffer->lines())
+	{
+		printf("QTermScreen::drawLine wrong index %d\n", index);
+		return startx; 
+	}
+
+	cstrDisplay = pTextLine->getText(startx);
+	cstrTest = pTextLine->getText();
+
+	if(m_pParam->m_nDispCode != m_pParam->m_nBBSCode)
+	{
+		char * chDisplay;
+		char * chTest;
+		if(m_pParam->m_nBBSCode==0) {
+			chDisplay = m_converter.G2B( cstrDisplay, cstrDisplay.length() );
+			chTest = m_converter.G2B( cstrTest, cstrTest.length() );
+		}
+
+		else {
+			chDisplay = m_converter.G2B( cstrDisplay, cstrDisplay.length() );
+			chTest = m_converter.G2B( cstrTest, cstrTest.length() );
+		}
+		
+		strDisplay = m_pCodec->toUnicode(chDisplay);
+		strTest = m_pCodec->toUnicode(chTest);
+
+		delete chTest;
+		delete chDisplay;
+	}
+	else {
+		strDisplay = m_pCodec->toUnicode(cstrDisplay);
+		strTest = m_pCodec->toUnicode(cstrTest);
+	}
+
+	if (strTest.find(strDisplay) == -1)
+		return startx - 1;
+	else
+		return startx;
+}
+
+		
 void QTermScreen::paintEvent( QPaintEvent * pe )
 {
 	
@@ -656,14 +863,13 @@ void QTermScreen::paintEvent( QPaintEvent * pe )
 	painter.begin( this );
 	
 	QRect rect = pe->rect().intersect(m_rcClient);
-	
+
 	QPoint tlPoint = mapToChar(QPoint(rect.left(), rect.top()));
 	
 	QPoint brPoint = mapToChar(QPoint(rect.right(),rect.bottom()));
 
 	for( int y=tlPoint.y(); y<=brPoint.y(); y++ )
 	{
-
 		drawLine(painter,y);
 		if( m_pBBS->isSelected(y)&&m_pParam->m_nMenuType==1 )
 		{
@@ -678,12 +884,12 @@ void QTermScreen::paintEvent( QPaintEvent * pe )
 
 	setUpdatesEnabled( true );
 
-	setMicroFocusHint((m_pBuffer->caret().x()+2)*m_nCharWidth,(m_pBuffer->caret().y()+1)*m_nCharHeight, 
-					m_nCharWidth, m_nCharHeight, true);
 }
 
-// draw a line with the specialAtter if given
-void QTermScreen::drawLine( QPainter& painter, int index)
+// draw a line with the specialAtter if given.
+// modified by hooey to draw part of the line.
+// I know I should not use starx, but I'm lazy ;)
+void QTermScreen::drawLine( QPainter& painter, int index, int starx, int endx, bool complete)
 {
 	if ( index >= m_pBuffer->lines())
 	{
@@ -703,20 +909,30 @@ void QTermScreen::drawLine( QPainter& painter, int index)
 	QCString cstrText;
 	QString strShow;
 	
-	drawMenuSelect(painter, index);
-	
-	for( uint i=0; i<linelength;i++)
+	if (starx < 0)
+		starx = 0;
+
+	if (endx > linelength || endx < 0)
+		endx = linelength;
+
+	if (complete == true) 
+		if (drawMenuSelect(painter, index)) {
+			starx = 0;
+			endx = linelength;
+		}
+
+	for( uint i=starx; i < endx;i++)
 	{
+		int offset = 0;
 		startx = i;
 		tempcp = color.at(i);
 		tempea = attr.at(i);
 		bSelected = m_pBuffer->isSelected(QPoint(i,index), m_pWindow->m_bCopyRect);
 		// get str of the same attribute
-		while ( tempcp == color.at(i) && 
+		while ( i < endx && tempcp == color.at(i) && 
 				tempea == attr.at(i)  && 
-				bSelected == m_pBuffer->isSelected(QPoint(i,index), m_pWindow->m_bCopyRect) && 
-				i < linelength )
-			i++;
+				bSelected == m_pBuffer->isSelected(QPoint(i,index), m_pWindow->m_bCopyRect) )
+			++i;
 
 		if(bSelected)	// selected area is text=color(0) background=color(7)
 			tempattr = SETCOLOR(SETFG(0)|SETBG(7)) | SETATTR(NO_ATTR);
@@ -740,9 +956,21 @@ void QTermScreen::drawLine( QPainter& painter, int index)
 		else
 			strShow = m_pCodec->toUnicode(cstrText);
 
-		drawStr(painter, strShow, startx, index, i - startx, tempattr, bSelected );
+		// Draw Charactors one by one to fix the variable font display problem
+		for (uint j=0; j < strShow.length(); ++j){
+			int length = 2;
+			if (strShow.at(j) < 0x7f) {
+				drawStr(painter, (QString)strShow.at(j), startx + offset, index, 1, tempattr, bSelected );
+				offset++;
+			}
+			else {
+				drawStr(painter, (QString)strShow.at(j), startx + offset, index, length, tempattr, bSelected );
+				offset += 2;
+			}
+		}
+		offset = 0;
 	
-		i--;
+		--i;
 	}
 }
 
@@ -822,8 +1050,11 @@ void QTermScreen::drawStr( QPainter& painter, const QString& str, int x, int y,
 		else
 			erase( mapToRect(x, y, length, 1) );
 	}
-	else
+	else {
+		if(GETBG(cp)!=0)
+			painter.fillRect( mapToRect(x,y,length,1), QBrush(m_color[GETBG(cp)]) );
 		painter.drawText( pt.x(), pt.y()+m_nCharAscent, str);
+	}
 }
 
 
@@ -839,7 +1070,7 @@ void QTermScreen::bossColor()
 	repaint(true);
 }	
 
-void QTermScreen::drawMenuSelect( QPainter& painter, int index )
+bool QTermScreen::drawMenuSelect( QPainter& painter, int index )
 {
 	QRect rcSelect, rcMenu, rcInter;
 	if( m_pBuffer->isSelected(index) )
@@ -892,7 +1123,9 @@ void QTermScreen::drawMenuSelect( QPainter& painter, int index )
 					painter.fillRect(rcMenu, QBrush(m_pParam->m_clrMenu));
 			break;
 		}
+		return true;
 	}
+	return false;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -942,4 +1175,192 @@ QRect QTermScreen::mapToRect( const QRect& rect )
 	return mapToRect( rect.x(), rect.y(), rect.width(), rect.height() );
 }
 
+// from KImageEffect::fade
+QImage& QTermScreen::fade( QImage& img, float val, const QColor& color)
+{
+    if (img.width() == 0 || img.height() == 0)
+      return img;
 
+    // We don't handle bitmaps
+    if (img.depth() == 1)
+	return img;
+
+    unsigned char tbl[256];
+    for (int i=0; i<256; i++)
+	tbl[i] = (int) (val * i + 0.5);
+
+    int red = color.red();
+    int green = color.green();
+    int blue = color.blue();
+
+    QRgb col;
+    int r, g, b, cr, cg, cb;
+
+    if (img.depth() <= 8) {
+	// pseudo color
+	for (int i=0; i<img.numColors(); i++) {
+	    col = img.color(i);
+	    cr = qRed(col); cg = qGreen(col); cb = qBlue(col);
+	    if (cr > red)
+		r = cr - tbl[cr - red];
+	    else
+		r = cr + tbl[red - cr];
+	    if (cg > green)
+		g = cg - tbl[cg - green];
+	    else
+		g = cg + tbl[green - cg];
+	    if (cb > blue)
+		b = cb - tbl[cb - blue];
+	    else
+		b = cb + tbl[blue - cb];
+	    img.setColor(i, qRgba(r, g, b, qAlpha(col)));
+	}
+
+    } else {
+	// truecolor
+        for (int y=0; y<img.height(); y++) {
+            QRgb *data = (QRgb *) img.scanLine(y);
+            for (int x=0; x<img.width(); x++) {
+                col = *data;
+                cr = qRed(col); cg = qGreen(col); cb = qBlue(col);
+                if (cr > red)
+                    r = cr - tbl[cr - red];
+                else
+                    r = cr + tbl[red - cr];
+                if (cg > green)
+                    g = cg - tbl[cg - green];
+                else
+                    g = cg + tbl[green - cg];
+                if (cb > blue)
+                    b = cb - tbl[cb - blue];
+                else
+                    b = cb + tbl[blue - cb];
+                *data++ = qRgba(r, g, b, qAlpha(col));
+            }
+        }
+    }
+
+    return img;
+}
+
+void QTermScreen::imStartEvent(QIMEvent * e)
+{
+	m_inputContent = new QTermInput(this, m_nCharWidth, m_nCharHeight, m_nCharAscent);
+	m_inputContent->setFont(*m_pFont);
+	//m_inputContent->setTextFormat(Qt::RichText);
+
+	m_inputContent->show();
+}
+
+void QTermScreen::imComposeEvent(QIMEvent * e)
+{
+	QString text = QString::null;
+
+	QPoint cursor;
+	int x = m_pBuffer->caretX();
+	int y = m_pBuffer->caretY();
+	int pos = e->cursorPos();
+
+	text += e->text();
+// how dirty
+#ifdef Q_OS_MACX
+ushort code=text[text.length()-1].unicode();
+if( (code>0xff00&&code<0xffef) || 
+(code>0x3000&&code<0x303f) || 
+(code>0x2000&&code<0x206f) ||
+code==0x00b7)
+{
+emit inputEvent(&text);
+return;
+}
+#endif
+	m_inputContent->drawInput(text, pos);
+/*
+	QString text_before = text.left(pos);
+	QString text_after = text.mid(pos + 1);
+	QString text_select = "<u>" + text.mid(pos,1) + "</u>";
+
+	text = text_before + text_select + text_after;
+*/
+	cursor = mapToPixel(QPoint(x+1,y));
+	
+	//m_inputContent->setText(text);
+	//m_inputContent->adjustSize();
+	
+	if (m_inputContent->width() + cursor.x() > m_rcClient.width()){
+		cursor = mapToPixel(QPoint(x,y));
+		cursor.setX(m_rcClient.right() - m_inputContent->width());
+	}else
+		cursor = mapToPixel(QPoint(x,y));
+	m_inputContent->move(cursor);
+}
+
+void QTermScreen::imEndEvent(QIMEvent * e)
+{
+	QString text = QString::null;
+	text += e->text();
+	//m_inputContent->setText(QString::null);
+	delete m_inputContent;
+	m_inputContent = NULL;
+	emit inputEvent(&text);
+}
+
+void QTermInput::drawInput(QString & inputText, int position)
+{
+	d_text = inputText;
+	d_pos = position;
+	repaint();
+}
+
+void QTermInput::paintEvent(QPaintEvent * e)
+{
+	QPainter inputPainter;
+	int len = 0;
+	int cursor = 0;
+	int width, height;
+	
+	if (d_pos == -1 || d_pos >= d_text.length())
+		d_pos = 0;
+	for (int i = 0; i < d_text.length(); ++i) {
+		if (d_text[i] <= 0x7f) {
+			++len;
+			if (i == d_pos)
+				cursor = len - 1;
+		}
+		else {
+			len += 2;
+			if (i == d_pos)
+				cursor = len - 2;
+		}
+	}
+	
+	width = len * d_width;
+	height = d_height;
+
+	inputPainter.begin(this);
+	setFixedSize(width,height);
+	erase();
+	
+	len = 0;
+	inputPainter.setPen(Qt::black);
+	//inputPainter.drawText(0 ,m_nCharAscent, text);
+
+	for (int j = 0; j < d_text.length(); ++j) {
+		inputPainter.drawText(len * d_width, d_ascent, d_text.mid(j, 1));
+		if (d_text[j] <= 0x7f)
+			++len;
+		else
+			len += 2;
+	}
+#ifndef Q_OS_MACX
+	QRect rcCurrent(cursor * d_width, 0, d_width*2, d_height);
+	if (d_text[d_pos] <= 0x7f)
+		rcCurrent.setRect(cursor * d_width, 0, d_width, d_height);
+	erase(rcCurrent);
+	inputPainter.fillRect(rcCurrent, QBrush(Qt::darkGray));
+	inputPainter.setPen(Qt::white);
+	inputPainter.drawText(cursor * d_width ,d_ascent, d_text.mid(d_pos, 1));
+#endif
+	inputPainter.end();
+}
+#include <qtermscreen.moc>
