@@ -4,6 +4,7 @@
 #include <qwidget.h>
 #include <qscrollbar.h>
 #include <qpixmap.h>
+#include <qlabel.h>
 #include "qtermconvert.h"
 
 class QTextCodec;
@@ -14,6 +15,25 @@ class QTermBuffer;
 class QTermBBS;
 class QTermParam;
 
+class QTermInput : public QWidget
+{
+	Q_OBJECT
+public:
+	QTermInput(QWidget * parent, int width, int height, int ascent)
+		:QWidget(parent),d_text(0),d_pos(0),d_height(height), d_width(width), d_ascent(ascent)
+	{
+	}
+	~QTermInput()
+	{
+	}
+	void drawInput(QString & text, int position);
+protected:
+	void paintEvent( QPaintEvent * );
+private:
+	QString d_text;
+	int d_pos;
+	int d_height, d_width, d_ascent;
+};
 
 class QTermScreen : public QWidget
 {
@@ -22,27 +42,20 @@ public:
 	QTermScreen( QWidget *parent, QTermBuffer *buffer, QTermParam *param, QTermBBS *bbs );
  	~QTermScreen();
 
-	const QColor * getColorTable();
-	void setColorTable( const QColor [] );
-	
-	QColor getBgColor();
-	void setBgColor( const QColor& );
-
-	QColor getFgColor();
-	void setFgColor( const QColor& );
-
+	void setSchema();
 
 	QFont getDispFont();
 	void setDispFont( const QFont& );
 	
 	void setBgPxm( const QPixmap& pixmap, int nType = 0);
 
-
 	void refreshScreen();
+	void blinkScreen();
 
 signals:
 	// 0 - enter  1 - press  2 - move  3 - release 4 - leave
 	void mouseAction( int, QMouseEvent * );
+	void inputEvent(QString *);
 
 public slots:
 	
@@ -51,7 +64,6 @@ public slots:
 	void updateScrollBar();
 
 protected:	
-	void initColorTable();
 	void initFontMetrics();
 
 	void moveEvent( QMoveEvent * );
@@ -70,11 +82,12 @@ protected:
 	void wheelEvent( QWheelEvent * );
 	
 	// display
+	int testChar(int, int);
 	void eraseRect( QPainter& , int, int, int, int, short );
 	void drawStr( QPainter&, const QString&, int, int, int, short, bool );
-	void drawLine( QPainter&, int index );
+	void drawLine( QPainter&, int index, int startx = -1, int endx = -1, bool complete = true);
 	void drawCaret( QPainter&, bool );
-	void drawMenuSelect( QPainter&, int );
+	bool drawMenuSelect( QPainter&, int );
 
 	// auxiluary
 	int getPosX( int xChar ) { return xChar*m_nCharWidth; }
@@ -87,6 +100,14 @@ protected:
 	QRect mapToRect( const QRect& );
 	
 	void updateFont();
+	void updateFont2();
+	void getFontMetrics(QFontMetrics *);
+
+	QImage& fade( QImage&, float, const QColor&);
+	void imStartEvent(QIMEvent * e);
+	void imComposeEvent(QIMEvent * e);
+	void imEndEvent(QIMEvent * e);
+	//void drawInput(QString &, int);
 
 protected slots:
 	void blinkEvent();
@@ -107,6 +128,7 @@ protected:
 	
 	QTimer * m_blinkTimer;
 	QTimer * m_cursorTimer;
+	QTermInput * m_inputContent;
 
 	bool m_blinkScreen;
 	bool m_blinkCursor;
@@ -122,6 +144,8 @@ protected:
 	
 	int m_nCharAscent,m_nCharDescent,m_nCharWidth,m_nCharHeight;
 	int m_nLineSpacing;	// for future
+
+	bool * m_pBlinkLine;
 	
 	// background
 	bool m_hasBg;
