@@ -11,24 +11,16 @@ AUTHOR:		smartfish kafa
 #include "qtermtelnet.h"
 #include "qterm.h"
 #include <stdio.h>
-#include <qsocket.h>
+#include "qtermsocket.h"
+#ifndef _NO_SSH_COMPILED
+#include "ssh/qtermsshsocket.h"
+#endif
 #include <qcstring.h>
-#include <malloc.h>
+
 #include <stdlib.h>
 #include <ctype.h>
+#include <qsocket.h>
 
-/* hack from wget/http.c */
-#define BASE64_LENGTH(len) (4 * (((len) + 2) / 3))
-static void
-base64_encode (const char *s, char *store, int length);
-static char *
-basic_authentication_encode (const char *user, const char *passwd,
-							     const char *header);
-static int
-parse_http_status_line (const char *line, const char **reason_phrase_ptr);
-
-
-const char QTermTelnet::wingate_enter = CTRL('J');
 
 struct fsm_trans QTermTelnet::ttstab[] = {
 	/* State	Input		Next State	Action	*/
@@ -55,9 +47,9 @@ struct fsm_trans QTermTelnet::ttstab[] = {
 	{ TSWOPT,	TOECHO,		TSDATA,		&QTermTelnet::do_echo		},
 	{ TSWOPT,	TONOGA,		TSDATA,		&QTermTelnet::do_noga		},
 	{ TSWOPT,	TOTXBINARY,	TSDATA,		&QTermTelnet::do_txbinary	},
-	{ TSWOPT,	TCANY,		TSDATA,		&QTermTelnet::do_notsup	},
+	{ TSWOPT,	TCANY,		TSDATA,		&QTermTelnet::do_notsup		},
 
-//	{ TSDOPT,	TONAWS,		TSDATA,		&QTermTelnet::will_naws	},
+	{ TSDOPT,	TONAWS,		TSDATA,		&QTermTelnet::will_naws		},
 	{ TSDOPT,	TOTERMTYPE,	TSDATA,		&QTermTelnet::will_termtype	},
 	{ TSDOPT,	TOTXBINARY,	TSDATA,		&QTermTelnet::will_txbinary	},
 	{ TSDOPT,	TCANY,		TSDATA,		&QTermTelnet::will_notsup	},
@@ -69,14 +61,14 @@ struct fsm_trans QTermTelnet::ttstab[] = {
 struct fsm_trans QTermTelnet::substab[] = {
         /* State        Input           Next State      Action  */
         /* ------       ------          -----------     ------- */
-	{ SS_START,	TOTERMTYPE,	SS_TERMTYPE,	&QTermTelnet::no_op		},
-	{ SS_START,	TCANY,		SS_END,		&QTermTelnet::no_op		},
+	{ SS_START,		TOTERMTYPE,	SS_TERMTYPE,	&QTermTelnet::no_op		},
+	{ SS_START,		TCANY,		SS_END,			&QTermTelnet::no_op		},
 
-	{ SS_TERMTYPE,	TT_SEND,	SS_END,		&QTermTelnet::subtermtype	},
-	{ SS_TERMTYPE,	TCANY,		SS_END,		&QTermTelnet::no_op		},
-
-	{ SS_END,	TCANY,		SS_END,		&QTermTelnet::no_op		},
-	{ FSINVALID,	TCANY,		FSINVALID,	&QTermTelnet::tnabort		},
+	{ SS_TERMTYPE,	TT_SEND,	SS_END,			&QTermTelnet::subtermtype	},
+	{ SS_TERMTYPE,	TCANY,		SS_END,			&QTermTelnet::no_op		},
+	
+	{ SS_END,		TCANY,		SS_END,			&QTermTelnet::no_op		},
+	{ FSINVALID,	TCANY,		FSINVALID,		&QTermTelnet::tnabort	},
 };
 
 
@@ -85,17 +77,17 @@ struct fsm_trans QTermTelnet::substab[] = {
  * Constructor
  *------------------------------------------------------------------------
  */
-QTermTelnet::QTermTelnet( QCString cstrTermType )
+QTermTelnet::QTermTelnet( QCString cstrTermType, int rows, int columns, bool isSSH, const char * sshuser, const char * sshpasswd )
 {
 	term = new char[21];
 	int i;
 	for(i=0;i<21;i++) term[i]='\000';	//clean up, need???
 	sprintf(term, cstrTermType);
 
-	wx=80;
-	wy=24;
+	wx=columns;
+	wy=rows;
+	
 	done_naws=0;
-
 	synching = 0;
 	doecho = 0;
 	sndbinary = 0;
@@ -103,11 +95,9 @@ QTermTelnet::QTermTelnet( QCString cstrTermType )
 	noga = 0;
 	termtype = 0;
 	naws = 0;
-	
-	// proxy related
-	proxy_type = NOPROXY;
-	proxy_state = 0;
-	bauth = false;
+
+	raw_size = 0;	
+
 	// init buffers
 	from_socket = new QByteArray();
 	to_ansi     = new QByteArray();
@@ -115,7 +105,13 @@ QTermTelnet::QTermTelnet( QCString cstrTermType )
 	to_socket   = new QByteArray();
 	
 	// create socket
-    	socket = new QSocket( this );
+	d_isSSH = isSSH;
+#ifndef _NO_SSH_COMPILED
+	if (d_isSSH)
+		socket = new QTermSSHSocket(sshuser, sshpasswd);
+	else
+#endif
+		socket = new QTermTelnetSocket();
 	
 	// connect signal and slots
 	connect ( socket, SIGNAL( connected() ),
@@ -130,6 +126,8 @@ QTermTelnet::QTermTelnet( QCString cstrTermType )
 		this, SLOT( delayCloseFinished() ) );
 	connect ( socket, SIGNAL( connectionClosed() ),
 		this, SLOT( closed() ) );
+	connect ( socket, SIGNAL( SocketState(int) ),
+		this, SIGNAL( TelnetState(int) ) );
 	// Init telnet, mainly the FSMs	
 	init_telnet();
 }
@@ -141,7 +139,7 @@ QTermTelnet::QTermTelnet( QCString cstrTermType )
 QTermTelnet::~QTermTelnet()
 {
 	// delete objects
-	delete term;
+	delete [] term;
     	delete socket;
 	delete from_socket;
 	delete to_ansi;
@@ -211,61 +209,41 @@ void QTermTelnet::fsminit(u_char fsm[][NCHRS], struct fsm_trans ttab[], int nsta
  */
 void QTermTelnet::connectHost(const QString& hostname, Q_UINT16 portnumber)
 {
-	host=hostname;
-	port=portnumber;
-	addr_host.sin_port=htons(portnumber);
-	
-	if( proxy_type == NOPROXY )
-	{
-		 socket->connectToHost( host, port );
-	}
-	else
-	{
-		//get ip addr from name
-		struct hostent * hostent;
-	#ifdef _OS_WIN32_	
-		WSADATA wsd;
-		if (WSAStartup(0x202,&wsd) != 0)
-		{
-			emit TelnetState(TSEINIWINSOCK);
-			WSACleanup( );
-			return;
-		}
-	#endif
-		hostent = gethostbyname( host );
-		if( hostent==NULL )
-		{
-			emit TelnetState(TSEGETHOSTBYNAME);
-			return;
-		}
-		//now only ipv4 support
-		memcpy(&addr_host.sin_addr,hostent->h_addr,4);
+	done_naws=0;
+	synching = 0;
+	doecho = 0;
+	sndbinary = 0;
+	rcvbinary = 0;
+	noga = 0;
+	termtype = 0;
+	naws = 0;
 
-		socket->connectToHost( proxy_host, proxy_port );
-	}
+	socket->connectToHost( hostname, portnumber );
 	// host name resolving
 	emit TelnetState( TSRESOLVING );
-
 }
 
-void QTermTelnet::setWindowSize( int x, int y )
+void QTermTelnet::windowSizeChanged(int x, int y)
 {
 	wx=x;
 	wy=y;
+	if(bConnected)
+	{
+		naws = 0;
 
-	char cmd[10];
-	
-	cmd[0] = (char)TCIAC;
-	cmd[1] = (char)TCSB;
-	cmd[2] = (char)TONAWS;
-	cmd[3] = (char)(short(x)>>8);
-	cmd[4] = (char)(short(x)%0xff);
-	cmd[5] = (char)(short(y)>>8);
-	cmd[6] = (char)(short(y)%0xff);
-	cmd[7] = (char)TCIAC;
-	cmd[8] = (char)TCSE;
+		char cmd[10];
+		cmd[0] = (char)TCIAC;
+		cmd[1] = (char)TCSB;
+		cmd[2] = (char)TONAWS;
+		cmd[3] = (char)(short(wx)>>8);
+		cmd[4] = (char)(short(wx)&0xff);
+		cmd[5] = (char)(short(wy)>>8);
+		cmd[6] = (char)(short(wy)&0xff);
+		cmd[7] = (char)TCIAC;
+		cmd[8] = (char)TCSE;
+		socket->writeBlock(cmd,9);
 
-	write(cmd, 9);
+	}
 }
 
 /*------------------------------------------------------------------------
@@ -276,15 +254,9 @@ void QTermTelnet::setProxy( int nProxyType, bool bAuth,
 			const QString& strProxyHost, Q_UINT16 uProxyPort,
 			const QString& strProxyUsr, const QString& strProxyPwd)
 {
-	proxy_type=nProxyType;
-	if ( proxy_type==NOPROXY )
-		return;
-	bauth		= bAuth;
-	proxy_host	= strProxyHost;
-	proxy_port	= uProxyPort;
-	proxy_usr	= strProxyUsr;
-	proxy_pwd	= strProxyPwd;
-
+	socket->setProxy(nProxyType, bAuth,
+			strProxyHost, uProxyPort,
+			strProxyUsr, strProxyPwd);
 }
 /*------------------------------------------------------------------------
  * close connection
@@ -304,81 +276,8 @@ void QTermTelnet::close()
  */
 void QTermTelnet::connected()
 {
-	QCString strPort;
-	char command[9];
-	memset(command,0,9);
-
-	char *proxyauth;
-	char *request;
-	int len=0;
-	
-	switch( proxy_type )
-	{
-	case NOPROXY:		// no proxy
-		emit TelnetState( TSHOSTCONNECTED );
-		return;
-	case WINGATE:		// Wingate Proxy
-		strPort.setNum( port );
-		write( host.local8Bit(), host.length() );
-		write( " ",1 );
-		write( strPort, strPort.length() );
-		write( &wingate_enter,1 );// CTRL+J
-		emit TelnetState( TSHOSTCONNECTED );
-		return;
-	case SOCKS4:		// Socks4 Proxy
-		command[0]='\x04';
-		command[1]='\x01';
-		memcpy(&command[2],&addr_host.sin_port,2);
-		memcpy(&command[4],&addr_host.sin_addr,4);
-		write(command,9);
-		proxy_state = 1;
-		emit TelnetState( TSPROXYCONNECTED );
-		return;
-	case SOCKS5:		// Socks5 Proxy
-		if( bauth )
-		{
-			command[0]='\x05';
-			command[1]='\x02';
-			command[2]='\x02';
-			command[3]='\x00';
-			write(command,4);
-		}
-		else
-		{
-			command[0]='\x05';
-			command[1]='\x01';
-			command[2]='\x00';
-			write(command,3);
-		}
-		proxy_state=1;
-		emit TelnetState( TSPROXYCONNECTED );
-		return;
-	case HTTP:
-		proxyauth=NULL;
-		if(bauth)
-			proxyauth = basic_authentication_encode
-				(proxy_usr, proxy_pwd, "Proxy-Authorization");
-
-		len = proxyauth!=NULL?strlen(proxyauth):0;
-	
-		request = new char[host.length()+len+81];
-		
-		sprintf(request,
-					"CONNECT %s:%u HTTP/1.0\n"
-					"%s\n", 
-					host.latin1(),port,
-					proxyauth!=NULL?proxyauth:"");
-
-		write(request, strlen(request));
-		delete request;
-		free(proxyauth);
-		proxy_state=1;
-		emit TelnetState( TSPROXYCONNECTED );
-		return;
-	default:
-		emit TelnetState( TSHOSTCONNECTED );
-		return;		
-	}		
+	bConnected = true;
+	emit TelnetState( TSHOSTCONNECTED );
 }
 /*------------------------------------------------------------------------
  * SLOT closed
@@ -387,6 +286,9 @@ void QTermTelnet::connected()
 
 void QTermTelnet::closed()
 {
+	bConnected = false;
+
+
 	emit TelnetState( TSCLOSED );
 }
 /*------------------------------------------------------------------------
@@ -396,6 +298,7 @@ void QTermTelnet::closed()
 
 void QTermTelnet::delayCloseFinished()
 {
+	bConnected = false;
 	emit TelnetState( TSCLOSEFINISH );
 }
 /*------------------------------------------------------------------------
@@ -441,17 +344,19 @@ void QTermTelnet::showError( int index )
 void QTermTelnet::socketReadyRead()
 {
 	int nbytes,nread;
-		
+
 	// get the data size
     	nbytes=socket->bytesAvailable();
 	if (nbytes <= 0)   return;
 		
+        raw_size=nbytes;
+	//qDebug("read size: %d", raw_size);
+
 	//resize input buffer
 	from_socket->resize(nbytes);
 		
 	//read data from socket to from_socket
 	nread=socket->readBlock( from_socket->data(), nbytes);
-	
 	//do some checks
 	if ( nread <= 0 ) {
 		qWarning("Reading from socket:nread<=0");
@@ -462,44 +367,6 @@ void QTermTelnet::socketReadyRead()
 		return;
 	}
 	
-	// Socks4 Proxy Reply
-	if( proxy_type==SOCKS4 && proxy_state==1 )
-	{
-		if( nread!=8 )
-		{
-			emit TelnetState( TSPROXYERROR );
-			return;
-		}
-
-		if( from_socket->at(1)==90 )
-			emit TelnetState( TSHOSTCONNECTED );
-		else
-			emit TelnetState( TSPROXYFAIL );
-		proxy_state=0;
-		return;
-	}
-
-	// HTTP Proxy Reply
-	if( proxy_type==HTTP && proxy_state!=0 )
-	{
-		const char *error;
-		int statcode = parse_http_status_line(from_socket->data(), &error);
-		if( statcode>= 200 && statcode<300 )
-			emit TelnetState( TSHOSTCONNECTED );
-		else
-			emit TelnetState( TSPROXYFAIL );
-		proxy_state=0;
-		return;
-	}
-
-	// Socks5 Proxy Reply
-	if( proxy_type==SOCKS5 && proxy_state!=0 )
-	{
-		socks5_reply(nread);
-		return;
-	}
-
-
 	//resize output buffer
 	to_ansi->resize(2*nread);
 	to_socket->resize(4*nread);
@@ -520,7 +387,6 @@ void QTermTelnet::socketReadyRead()
 		ttstate = pt->ft_next;
 	}
 	
-	
 	// flush the to_socket buffer, it contain response to server
 	if ( wsize > 0 ){
 		socket->writeBlock(to_socket->data(), wsize);
@@ -528,11 +394,39 @@ void QTermTelnet::socketReadyRead()
 	}
 										
 	/* send SIGNAL readyRead() with the size of data available*/					
-	if ( rsize > 0 )
+	if ( rsize > 0 || raw_size > 0)
 		emit readyRead(rsize);	
 			
 }
 
+int QTermTelnet::raw_len()
+{
+    return raw_size;
+}
+
+/*------------------------------------------------------------------------
+ * actions
+ *------------------------------------------------------------------------
+ */
+int QTermTelnet::read_raw(char * data, uint maxlen)
+{
+    //do some checks
+    if(data==0) {
+        qWarning("read: NULL pointer");
+        return -1;
+    }
+    if(maxlen < raw_size) {
+        /* we need all data be read out in one read */
+        qWarning("read: upper layer accept buffer too small");
+        return -1;
+    }
+
+    //do it, memcpy( destination, source, size)
+    memcpy(data, from_socket->data(), raw_size);
+    return raw_size;
+}
+
+
 /*------------------------------------------------------------------------
  * actions
  *------------------------------------------------------------------------
@@ -592,8 +486,9 @@ int QTermTelnet::write(const char * data, uint len)
 	socket->writeBlock(to_socket->data(), wsize);
 	socket->flush();	
 
-	return 0;
+	emit TelnetState(TSWRITED);
 
+	return 0;
 }
 
 
@@ -829,29 +724,35 @@ int QTermTelnet::will_termtype(int c)
 
 int QTermTelnet::will_naws(int c)
 {
-	done_naws = 1;
-
 	if (naws) {
 		if (option_cmd == TCDO)
 			return 0;
 	} else if (option_cmd == TCDONT)
 		return 0;
 
+
 	naws = !naws;
 
+	putc_down(TCIAC);
 	if (naws)
-	{
-		if(!done_naws)
-		{
-			putc_down(TCIAC);
-			putc_down(TCWILL);
-			putc_down((char)c);
-		}
-		setWindowSize( wx, wy );
-	}
-	
+		putc_down(TCWILL);
+	else
+		putc_down(TCWONT);
+	putc_down((char)c);
+
+	putc_down(TCIAC);
+	putc_down(TCSB);
+	putc_down(TONAWS);
+	putc_down((char)(short(wx)>>8));
+	putc_down((char)(short(wx)&0xff));
+	putc_down((char)(short(wy)>>8));
+	putc_down((char)(short(wy)&0xff));
+	putc_down(((char)TCIAC));
+	putc_down((char)TCSE);
+
 	return 0;
 }
+
 /*------------------------------------------------------------------------
  * subopt - do option subnegotiation FSM transitions
  *------------------------------------------------------------------------
@@ -1013,244 +914,5 @@ int QTermTelnet::tnabort(int)
 //	exit(-1);
 	return -1;
 }
-/*------------------------------------------------------------------------
- * connect command for socks5
- *------------------------------------------------------------------------
- */
-void QTermTelnet::socks5_connect()
-{
-	char * command = new char[10];
-	memset(command,0,10);
-	command[0]='\x05';
-	command[1]='\x01';
-	command[2]='\x00';
-	command[3]='\x01';
-	memcpy( command+4, &addr_host.sin_addr,4 );
-	memcpy( command+8, &addr_host.sin_port,2 ); 
-	write( command, 10 );
-	delete []command;
-}
-/*------------------------------------------------------------------------
- * authentation command for socks5
- *------------------------------------------------------------------------
- */
-void QTermTelnet::socks5_auth()
-{
-	int ulen = proxy_usr.length();
-	int plen = proxy_pwd.length();
-	char * command = new char[3+ulen+plen];
-	sprintf((char *)command,"  %s %s",proxy_usr.local8Bit().data(),
-					proxy_pwd.local8Bit().data());
-	command[0]='\x01';
-	command[1]=ulen;
-	command[2+ulen] = plen;
-	write( command,3+ulen+plen );
-	delete []command;	
-}
-/*------------------------------------------------------------------------
- * reply  from socks5
- *------------------------------------------------------------------------
- */
-void QTermTelnet::socks5_reply( int nread )
-{
-	if( proxy_state==1 )	// Socks5 Proxy Replay 1
-	{
-		if( nread!=2 )
-		{
-			proxy_state=0;
-			emit TelnetState( TSPROXYERROR );
-			return;
-		}
-		switch( from_socket->at(1) )
-		{
-		case '\x00':	// no authentation needed
-			socks5_connect();
-			proxy_state=3;
-			emit TelnetState( TSCONNECTVIAPROXY );
-			return;
-		case '\x02':	//need user/password
-			socks5_auth();
-			proxy_state=2;
-			emit TelnetState( TSPROXYAUTH );
-			return;
-		case '\xff':
-			proxy_state=0;
-			emit TelnetState( TSPROXYFAIL );
-			return;
-		default:
-			proxy_state=0;
-			emit TelnetState( TSPROXYFAIL );
-			return;
-		}			
-	}
 
-	else if( proxy_state==2)	//Socks5 Proxy Replay 2
-	{
-		if( nread!=2 )
-		{
-			proxy_state=0;
-			emit TelnetState( TSPROXYERROR );
-			return;
-		}
-		
-
-		if( from_socket->at(1)!='\00')
-		{
-			proxy_state=0;
-			emit TelnetState( TSPROXYFAIL );
-			return;
-		}
-		socks5_connect();
-		proxy_state=3;
-		emit TelnetState( TSCONNECTVIAPROXY );
-		return;
-	}
-	else if( proxy_state==3)	//Socks5 Proxy Replay 3
-	{
-		proxy_state=0;
-		if( nread!=10 )
-		{
-			emit TelnetState( TSPROXYERROR );
-			return;
-		}
-		if( from_socket->at(1)!='\00' )
-		{
-			emit TelnetState( TSPROXYFAIL );
-			return;
-		}		
-		emit TelnetState( TSHOSTCONNECTED );
-		return;	
-	}	
-	else
-	{
-		proxy_state=0;
-		return;
-	}
-}
-
-/* hack from wget/http.c */
-/* How many bytes it will take to store LEN bytes in base64.  */
-#define BASE64_LENGTH(len) (4 * (((len) + 2) / 3))
-
-/* Encode the string S of length LENGTH to base64 format and place it
-   to STORE.  STORE will be 0-terminated, and must point to a writable
-   buffer of at least 1+BASE64_LENGTH(length) bytes.  */
-static void
-base64_encode (const char *s, char *store, int length)
-{
-  /* Conversion table.  */
-  static char tbl[64] = {
-    'A','B','C','D','E','F','G','H',
-    'I','J','K','L','M','N','O','P',
-    'Q','R','S','T','U','V','W','X',
-    'Y','Z','a','b','c','d','e','f',
-    'g','h','i','j','k','l','m','n',
-    'o','p','q','r','s','t','u','v',
-    'w','x','y','z','0','1','2','3',
-    '4','5','6','7','8','9','+','/'
-  };
-  int i;
-  unsigned char *p = (unsigned char *)store;
-
-  /* Transform the 3x8 bits to 4x6 bits, as required by base64.  */
-  for (i = 0; i < length; i += 3)
-    {
-      *p++ = tbl[s[0] >> 2];
-      *p++ = tbl[((s[0] & 3) << 4) + (s[1] >> 4)];
-      *p++ = tbl[((s[1] & 0xf) << 2) + (s[2] >> 6)];
-      *p++ = tbl[s[2] & 0x3f];
-      s += 3;
-    }
-  /* Pad the result if necessary...  */
-  if (i == length + 1)
-    *(p - 1) = '=';
-  else if (i == length + 2)
-    *(p - 1) = *(p - 2) = '=';
-  /* ...and zero-terminate it.  */
-  *p = '\0';
-}
-
-/* Create the authentication header contents for the `Basic' scheme.
-   This is done by encoding the string `USER:PASS' in base64 and
-   prepending `HEADER: Basic ' to it.  */
-static char *
-basic_authentication_encode (const char *user, const char *passwd,
-			     const char *header)
-{
-  char *t1, *t2, *res;
-  int len1 = strlen (user) + 1 + strlen (passwd);
-  int len2 = BASE64_LENGTH (len1);
-
-  t1 = (char *)alloca (len1 + 1);
-  sprintf (t1, "%s:%s", user, passwd);
-  t2 = (char *)alloca (1 + len2);
-  base64_encode (t1, t2, len1);
-  res = (char *)malloc (len2 + 11 + strlen (header));
-  sprintf (res, "%s: Basic %s\r\n", header, t2);
-
-  return res;
-}
-
-/* Parse the HTTP status line, which is of format:
-
-   HTTP-Version SP Status-Code SP Reason-Phrase
-
-   The function returns the status-code, or -1 if the status line is
-   malformed.  The pointer to reason-phrase is returned in RP.  */
-static int
-parse_http_status_line (const char *line, const char **reason_phrase_ptr)
-{
-  /* (the variables must not be named `major' and `minor', because
-     that breaks compilation with SunOS4 cc.)  */
-  int mjr, mnr, statcode;
-  const char *p;
-
-  *reason_phrase_ptr = NULL;
-
-  /* The standard format of HTTP-Version is: `HTTP/X.Y', where X is
-     major version, and Y is minor version.  */
-  if (strncmp (line, "HTTP/", 5) != 0)
-    return -1;
-  line += 5;
-
-  /* Calculate major HTTP version.  */
-  p = line;
-  for (mjr = 0; isdigit (*line); line++)
-    mjr = 10 * mjr + (*line - '0');
-  if (*line != '.' || p == line)
-    return -1;
-  ++line;
-
-  /* Calculate minor HTTP version.  */
-  p = line;
-  for (mnr = 0; isdigit (*line); line++)
-    mnr = 10 * mnr + (*line - '0');
-  if (*line != ' ' || p == line)
-    return -1;
-  /* Wget will accept only 1.0 and higher HTTP-versions.  The value of
-     minor version can be safely ignored.  */
-  if (mjr < 1)
-    return -1;
-  ++line;
-
-  /* Calculate status code.  */
-  if (!(isdigit (*line) && isdigit (line[1]) && isdigit (line[2])))
-    return -1;
-  statcode = 100 * (*line - '0') + 10 * (line[1] - '0') + (line[2] - '0');
-
-  /* Set up the reason phrase pointer.  */
-  line += 3;
-  /* RFC2068 requires SPC here, but we allow the string to finish
-     here, in case no reason-phrase is present.  */
-  if (*line != ' ')
-    {
-      if (!*line)
-	*reason_phrase_ptr = line;
-      else
-	return -1;
-    }
-  else
-    *reason_phrase_ptr = line + 1;
-
-  return statcode;
-}
+#include <qtermtelnet.moc>
