@@ -67,7 +67,7 @@ void QTermTextLine::insertText( const QCString& str, short attribute, int index
 	else			// insert
 	{
 
-		if(index>m_length)
+		if(index>=m_length)
 		{
 			m_text.insert( index, str );
 			m_length = m_text.length();		
@@ -137,7 +137,7 @@ void QTermTextLine::replaceText( const QCString& str, short attribute, int index
 	if ( len == -1 )	// replace with  str
 		len = newlen;
 
-	if( index>m_length )
+	if( index>=m_length )
 	{
 			tmp.fill( ' ', index-m_length );
 			m_text.replace( index, len, str );
@@ -211,7 +211,7 @@ QCString QTermTextLine::getText( int index, int len )
 	if ( index == -1 )
 		str = m_text;
 	else if ( len == -1 )	
-		str = m_text.right( index );
+		str = m_text.mid( index, len );
 	else
 		str = m_text.mid( index, len );
 
@@ -233,7 +233,12 @@ QCString QTermTextLine::getAttrText( int index, int len, const QCString& escape
 	else if ( len == -1 )
 		len = m_length-index;
 
-
+	
+	if(index>=m_length)
+		return (char *)NULL;
+		
+	printf("index=%d len=%d m_length=%d\n", index, len, m_length);
+	
 	for( int i=index; i<index+len && i<m_length; i++)
 	{
 		startx = i;
