@@ -26,10 +26,11 @@ QTermTimeLabel::QTermTimeLabel(QWidget * parent, const char * name, WFlags f)
 //destructor
 QTermTimeLabel::~QTermTimeLabel()
 {
-	
+	delete currentTime;	
 }
 //timer to display the current time
 void QTermTimeLabel::timerEvent(QTimerEvent*)
 {
 	setText(currentTime->currentTime().toString());	
 }
+#include <qtermtimelabel.moc>
