@@ -8,8 +8,9 @@ AUTHOR:        kingson fiasco
                                     NOTE
  This file may be used, distributed and modified without limitation.
  *******************************************************************************/
-#include "qtermwndmgr.h"
+
 #include "qtermwindow.h"
+#include "qtermwndmgr.h"
 #include "qtermframe.h"
 
 #include <qapplication.h>
@@ -23,6 +24,8 @@ AUTHOR:        kingson fiasco
 #include <qlist.h>
 #endif
 
+#include <qglobal.h>
+
 #include <stdio.h>
 
 extern QString pathLib;
@@ -35,6 +38,9 @@ QTermWndMgr::QTermWndMgr( QObject * parent, const char * name )
 	pFrame = (QTermFrame *) parent;
 	
 	nActive = -1;
+	
+	removed = false;
+
 	pWin.setAutoDelete( false );
 	pTab.setAutoDelete( false );
 	pIcon.setAutoDelete( false );
@@ -79,6 +85,7 @@ void QTermWndMgr::removeWindow(QTermWindow * mw)
 		pFrame->enableMenuToolBar( false );
 	}
 	
+	removed = true;
 	//remove from the Tabbar
 	pFrame->tabBar->removeTab(qtab);	
 }
@@ -88,6 +95,12 @@ void QTermWndMgr::activateTheTab(QTermWindow * mw)
 	//find where it is
 	int n=pWin.find(mw);
 
+	if( n==nActive )
+		return;
+	
+	mw->showNormal();
+	mw->showMaximized();
+	
 	nActive = n;
 
 	QTab * qtab=pTab.at(n);
@@ -100,14 +113,23 @@ void QTermWndMgr::activateTheTab(QTermWindow * mw)
 //active the window when switch the tab
 void QTermWndMgr::activateTheWindow(QTab *qtab)
 {
+
 	//find where it is
 	int n=pTab.find(qtab);
 
+	if( n==nActive )
+		return;
+
 	nActive = n;
 
 	QTermWindow * mw=pWin.at(n);
 	//set focus to it
-	mw->setFocus();	
+//	#if (QT_VERSION>=0x030300)
+	((QWidget*)pFrame->ws)->setFocus();
+	mw->showNormal();
+//	#else
+//	mw->setFocus();
+//	#endif
 	
 	pFrame->updateMenuToolBar();
 }
@@ -119,9 +141,9 @@ void QTermWndMgr::blinkTheTab(QTermWindow * mw,bool bVisible)
 	QIconSet* icon=pIcon.at(n);
 		
 	if(bVisible)
-		icon->setPixmap(pathLib+"./pic/tabpic.xpm",QIconSet::Automatic);
+		icon->setPixmap(pathLib+"pic/tabpad.png",QIconSet::Automatic);
 	else
-		icon->setPixmap(pathLib+"./pic/blink.xpm",QIconSet::Automatic);
+		icon->setPixmap(pathLib+"pic/transp.png",QIconSet::Automatic);
 
 	pFrame->tabBar->update();		
 }
@@ -151,5 +173,31 @@ void QTermWndMgr::activeNextPrev(bool next)
 		n = (n==0)?pWin.count()-1:n-1;
 	
 	nActive = n;
-	pWin.at(n)->setFocus();
+
+	QTermWindow * mw=pWin.at(n);
+	//set focus to it
+//	#if (QT_VERSION>=0x030300)
+	((QWidget*)pFrame->ws)->setFocus();
+	mw->showNormal();
+//	#else
+//	mw->setFocus();
+//	#endif
+
+	QTab * qtab=pTab.at(n);
+	//set it seleted
+	pFrame->tabBar->setCurrentTab(qtab);
+	pFrame->updateMenuToolBar();
+
 }
+
+bool QTermWndMgr::afterRemove()
+{
+	if(removed) 
+	{
+		removed=false;
+		return true;
+	}
+	else
+		return false;
+}	
+#include <qtermwndmgr.moc>
